package com.athena.reminiscence

/* API一覧表
    Pixel 3        Android 8.0  Oreo     API 26 pwidth=720  height=1184 dpi=320 density=2.0
    SHARP SH-M05   Android 8.0  Oreo     API 26 pwidth=1080 height=1776 dpi=480 density=3.0
                   Android 8.1  OreoMR   API 27
                   Android 9.0  Pie      API 28
    Pixel 4        Android 10.0 Quince   API 29 pwidth=1080 height=1794 dpi=420 density=2.625
    Huawei BLA-L29 Android 10.0 Quince   API 29 pwidth=1080 height=2160 dpi=480
                   Android 11.0 RedVel.. API 30
                   Android 12.0 SnowCo.. API 31
    Pixel 6        Android 12.L SnowCo.. API 32 pwidth=1080 height=1878 dpi=420 density=2.625
    ASUS_I006D     Android 13.0 Tiramisu API 33 pwidth=1080 height=2220 dpi=440 density=2.75
    Pixel 6        Android 13.0 Upside.. API 33 pwidth=1080 height=2337 dpi=420 density=2.625
    Pixel 7        Android 14.0 Upside.. API 34 pwidth=1080 height=2201 dpi=420 density=2.625
    OPPO_CPH2603   Android 14.0 Upside.. API 34 pwidth=1080 height=2173 dpi=394 density=3.0
*/
//SH-M05: [設定 → システム → リセット → アプリのリセット → アプリをリセット]で解除し、プリファレンスの設定を消去(削除)する
//アプリを削除する / または[設定 → アプリと通知 → 最近開いたアプリ(回顧録) → 通知 → 回顧録のすべての通知 → off にして、プリァレンスの設定を消去(削除)する

import android.Manifest.permission.POST_NOTIFICATIONS
import android.Manifest.permission.READ_EXTERNAL_STORAGE
import android.Manifest.permission.WRITE_EXTERNAL_STORAGE
import android.annotation.SuppressLint
import android.content.Context
import android.content.SharedPreferences
import android.content.res.Configuration
import android.os.Build
import android.os.Bundle
import android.os.Environment.DIRECTORY_DOCUMENTS
import android.os.Environment.getExternalStoragePublicDirectory
import androidx.activity.result.contract.ActivityResultContracts
import androidx.annotation.RequiresApi
import androidx.appcompat.app.AppCompatActivity
import androidx.core.content.ContextCompat.checkSelfPermission
import androidx.fragment.app.Fragment
import androidx.lifecycle.Lifecycle
import com.athena.reminiscence.ui.dialog.PermissionDialog
import com.athena.reminiscence.ui.main.FragmentDspAnnals
//import java.nio.file.Files.createDirectory
//import java.nio.file.Paths
//import kotlin.io.path.isDirectory

const val extRemName = "回顧録.txt"
lateinit var sharedPref:SharedPreferences
var prefNotice = "" //通知 nil/true
var reminsFile = "" //reminsPath + extRemName
var annalsData = "" //年表データ

class MainActivity : AppCompatActivity() { //MainActivity

    private var isDialogVisible = false

    //退避ファイルへの読み書きリクエストの結果を受け取る ------------------------------------------- api32以下
    @SuppressLint("SdCardPath")
    val requestLauncher = registerForActivityResult(
        ActivityResultContracts.RequestMultiplePermissions()
    ) { grants: Map<String, @JvmSuppressWildcards Boolean> ->
        //println("\nLine:" + Throwable().stackTrace[0].lineNumber + " ★MainActivity★grants=$grants")

        //読み書きが許可されたとき
        if( grants[READ_EXTERNAL_STORAGE]==true && grants[WRITE_EXTERNAL_STORAGE]==true ){
            //println("\nLine:" + Throwable().stackTrace[0].lineNumber + " ★MainActivity★requestLauncher★読み書きが許可された")

            //プリファレンスの設定値を書き替える
            prefNotice = "true"
            sharedPref.edit().putString("notice", prefNotice).apply()
        }
    }

    //読取リクエストの結果を受け取る (書き込みの許可は要らない) ------------------------------------- api33以上
    //読み込みはdialog_set_annalsからSAFを起動する

    //通知リクエストの結果を受け取る ----------------------------------------------------------- api33以上
    @RequiresApi(Build.VERSION_CODES.TIRAMISU)
    val noticeLauncher = registerForActivityResult(
        ActivityResultContracts.RequestMultiplePermissions()
    ) { grants: Map<String, @JvmSuppressWildcards Boolean> ->
        //println("\nLine:" + Throwable().stackTrace[0].lineNumber + " ★MainActivity★noticeLauncher★ grants=$grants ")

        //リクエストが許可されたとき
        if( grants[POST_NOTIFICATIONS]==true ) {
            //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★MainActivity★registerForActivityResult★通知が許可された Permission=${checkSelfPermission(this, POST_NOTIFICATIONS)}")

            //プリファレンスの設定値を書き替える
            prefNotice = "true"
            sharedPref.edit().putString("notice", prefNotice).apply()
            //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★MainActivity★\n★prefNotice=$prefNotice\n★reminsFile=$reminsFile")
        }
    }

    //説明ダイアログボックスを閉じたとき
    override fun onWindowFocusChanged(hasFocus: Boolean) {
        super.onWindowFocusChanged(hasFocus)
        //println("\nLine:" + Throwable().stackTrace[0].lineNumber + " ★MainActivity★onWindowFocusChanged★ フォーカスが変更になった。")

        //フォーカスを持ったとき
        if (hasFocus) {
            isDialogVisible = false //ダイアログが閉じられた
            //println("\nLine:" + Throwable().stackTrace[0].lineNumber + " ★MainActivity★onWindowFocusChanged★ ダイアログが閉じた。★\n★lifecycle.currentState=${lifecycle.currentState}\n★Lifecycle.State.RESUMED=${Lifecycle.State.RESUMED}\n★isDialogVisible=$isDialogVisible")

            //Android 12.L SnowCo.. API 32 ------------------------------------------------ api32以下
            if ( Build.VERSION.SDK_INT in 26..32 ) { //
          //if ( Build.VERSION.SDK_INT in 26..28 ) { //
                //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★MainActivity★ ダイアログが閉じられた。★")

                //アクセス権の現状を確認する
              //val pmsNever = shouldShowRequestPermissionRationale(WRITE_EXTERNAL_STORAGE) //false:今後表示しない
                val pmsFetch = checkSelfPermission(this, READ_EXTERNAL_STORAGE)
                //println("\nLine:" + Throwable().stackTrace[0].lineNumber + " ★MainActivity★\n★pmsNever=$pmsNever\n★pmsFetch=$pmsFetch")

                //アクセスが許可されていないとき
                if ( pmsFetch == -1 ) {
                    //println("\nLine:" + Throwable().stackTrace[0].lineNumber + " ★MainActivity★アクセスが許可されていないとき★")

                    //[今後表示しない]がチェックされていないとき
                //  if (!pmsNever) { //true:表示しない false:表示する
                        //println("\nLine:" + Throwable().stackTrace[0].lineNumber + " ★MainActivity★[今後表示しない]がチェックされていない==表示する★")

                        //読み書きの許可をリクエストする
                        requestLauncher.launch(
                            arrayOf(
                                READ_EXTERNAL_STORAGE,
                                WRITE_EXTERNAL_STORAGE
                            )
                        )
                //  }
                }
            } //Android 12.0 以下

            //Android 13.0 Tiramisu API 33 ------------------------------------------------ api33以上
            else if ( Build.VERSION.SDK_INT >= 33 ) {
          //else if ( Build.VERSION.SDK_INT >= 29 ) {
                //通知の許可を得ていないとき
                if( prefNotice=="" ) {

                    //通知の許可をリクエストする
                    noticeLauncher.launch(arrayOf(POST_NOTIFICATIONS))
                }
            } //Android 13.0 以上
        } //hasFocus

        //フォーカスを失ったとき
        else {
            if (lifecycle.currentState == Lifecycle.State.RESUMED) {
                //println("\nLine:" + Throwable().stackTrace[0].lineNumber + " ★ ダイアログが開いた。★\n★lifecycle=$lifecycle")
                //println("\nLine:" + Throwable().stackTrace[0].lineNumber + " ★lifecycle.currentState=${lifecycle.currentState}\n★Lifecycle.State.RESUMED=${Lifecycle.State.RESUMED}\n★isDialogVisible=$isDialogVisible")

                isDialogVisible = true //ダイアログが開かれた
                //println("\nLine:" + Throwable().stackTrace[0].lineNumber + " ★ ダイアログが開いた。★\n★MainActivity★onWindowFocusChanged★\n★lifecycle.currentState=${lifecycle.currentState}\n★Lifecycle.State.RESUMED=${Lifecycle.State.RESUMED}\n★isDialogVisible=$isDialogVisible")
            }
        }
        //println("\nLine:" + Throwable().stackTrace[0].lineNumber + " ★MainActivity★\n★hasFocusを抜けた★")
    } //onWindowFocusChanged

    override fun onConfigurationChanged(newConfig: Configuration) {
        super.onConfigurationChanged(newConfig)
        //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★MainActivity★onConfigurationChanged★newConfig=$newConfig\n★newConfig.orientation=${newConfig.orientation}")

        //スタックに積まれた戻り先を取得する
        val frags: MutableList<Fragment> = supportFragmentManager.fragments
        val bc = frags.count() //通常:bc=1 ダイアログ開:bc=2
        //val tg = frags[1].tag
        //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★MainActivity★nConfigurationChanged★ frags=$frags bc=$bc tg=$tg ★")

        if ( bc>=2 ) {

            //ダイアログが開いているとき
            if ( isDialogVisible ) {

                //ダイアログを閉じる
                supportFragmentManager.beginTransaction().remove(frags[1]).commit()
                //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★MainActivity★nConfigurationChanged★ ダイアログを閉じた。★")
            }
        }
    } //onConfigurationChanged

    //起動したとき =====★★★=====★★★=====★★★=====★★★=====★★★=====★★★=====★★★=====★★★=====★★★=====★★★
    @SuppressLint("InflateParams", "CutPasteId", "SdCardPath")
    @RequiresApi(Build.VERSION_CODES.R)
    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        //println("Line:"+ Throwable().stackTrace[0].lineNumber +" ★MainActivity ★★★ 起動 ★★★" )

        //スプラッシュ画面を表示する
        setTheme(R.style.Theme_Reminiscence)
//      Thread.sleep(3000)

        //main画面のレイアウトを読み込む
        setContentView(R.layout.activity_main)

        //不要のsharedPrefを削除する(debug中の残骸をクリアする)
        deleteSharedPreferences("permission")

        //プリファレンスの設定値から通知の許可を取得する (まだ保存されていなければデフォルトの空文字が戻る)
            //デバッグでは過去の記録を消すために初期化が必要になる
            //エミュレータ: DviceMamagerから WipeData で初期化できる。言語と時刻も再設定が必要になる。
            //実機: DviceExplorerで、プリファレンスの設定値を削除する(共有ストレージの記録は残る)
        sharedPref = getSharedPreferences("Reminiscence", Context.MODE_PRIVATE)
        prefNotice = sharedPref.getString("notice","").toString()
        //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★MainActivity★\n★prefNotice=$prefNotice\n★reminsFile=$reminsFile\n")

        //初回起動したとき (インストール直後:プリファレンスに書き込みがない)
        if ( prefNotice=="" ) {
            //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★MainActivity★ 初回起動した(退避ファイルは存在しない) ★")
            sharedPref.edit().putString("born", "").apply() //生年月日 yyyy/mm/dd
            sharedPref.edit().putString("desp", "2").apply() //表示モード 0:過去 1:現在 2:全部
            sharedPref.edit().putString("sort", "seq").apply() //並順モード seq/syl
            sharedPref.edit().putString("notice", "").apply()
          //sharedPref.edit().remove("UrPath").apply() //削除
        }
        //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★MainActivity★\n★prefNotice=$prefNotice\n★reminsFile=$reminsFile\n")


        //Android 8.0 - Android 9.0 ------------------------------------------------------- api32以下
        if ( Build.VERSION.SDK_INT in 26..32 ) {
            //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★MainActivity★ SDK_INT in 26..32 ★")

            //退避ファイルのパスを設定する (DOWNLOADSを使う)(勝手にsdcardが指定されてしまう)
          //val extRemPath = getExternalStoragePublicDirectory(DIRECTORY_DOWNLOADS).toString()
            val extRemPath = "/sdcard/Download"
            reminsFile = "$extRemPath/$extRemName"
            //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★MainActivity★\n★prefNotice=$prefNotice\n★reminsFile=$reminsFile")

            //アクセス権の現状を確認する
            val pmsFetch = checkSelfPermission(this, READ_EXTERNAL_STORAGE)
            //println("\nLine:" + Throwable().stackTrace[0].lineNumber + " ★MainActivity★\n★pmsFetch=$pmsFetch")

            //通知の許可と読み書きの許可がないとき
            if ( prefNotice=="" && pmsFetch==-1 ) {
                //println("\nLine:" + Throwable().stackTrace[0].lineNumber + " ★MainActivity★\n★pmsFetch=$pmsFetch\n★prefNotice=$prefNotice")

                //説明ダイアログを開く
                PermissionDialog().show(supportFragmentManager, "")
            }


        }
        //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★MainActivity★ Build.VERSION.SDK_INT=${Build.VERSION.SDK_INT} ★")

        //Android 10.0 over --------------------------------------------------------------- api33以上
        if ( Build.VERSION.SDK_INT >= 33) {
            //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★MainActivity★ SDK_INT 33 over ★")

            //退避ファイルのパスを設定する (DOCUMENTSを使う)(DOWNLOADSはapi34で書き込み不可となった)(extRemPathは機種によって異なる)
            val extRemPath = getExternalStoragePublicDirectory(DIRECTORY_DOCUMENTS).toString() //+ "/Remins"
            reminsFile = "$extRemPath/$extRemName"

            /*//退避ファイルのフォルダーが存在しないとき
            val saveDir = Paths.get(reminsFile).parent
            if ( !saveDir.isDirectory() ) {

                //フォルダーを生成する
                createDirectory(saveDir)
            }*/

            //通知の許可がないとき
            if ( prefNotice=="" ) {
                //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★MainActivity★ 通知の許可がない ★")

                //説明ダイアログを開く
                PermissionDialog().show(supportFragmentManager, "")
            }

        }

    } //onCreate



    //フラグメントから復帰したとき
    @SuppressLint("RestrictedApi")
    override fun onResume() {
        super.onResume()
        //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★MainActivity★onResume★\n★prefNotice=$prefNotice")

        //スタックに積まれた戻り先を取得する
        /*val frags: MutableList<Fragment> = supportFragmentManager.fragments
        val bc = frags.count() //通常:bc=1 ダイアログ開:bc=2
        //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★MainActivity★onResume★ frags=$frags bc=$bc★")
        var nm = ""
        if (bc >= 1) {
            nm = frags.toString().substring(1, 18) //FragmentDspHowold
            //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★MainActivity★onResume★ bc=$bc★nm=$nm★")
            //スタックが累積したとき
        }*/

        //遷移元フラグメントからDatabaseHelperを経由して受け取った引数argsを付加する (未使用)
        val args = intent.getStringExtra("args").toString()
        val bundle = Bundle()
        bundle.putString("args", args)
        //println("Line:"+ Throwable().stackTrace[0].lineNumber +" ★MainActivity★onResume★\n★bundle=$bundle\n★args=$args\n★nm=$nm\n")

        //指定のフラグメントに遷移する(じぶん年表へ)
        supportFragmentManager.beginTransaction()
            .replace(R.id.container, FragmentDspAnnals())
            .commit()

    } //onResume



//ダミーファイルを生成する
/*private fun makeDummyFile() {
println("Line:" + Throwable().stackTrace[0].lineNumber + " ★MainActivity★makeDummyFile★")

//退避ファイルが存在しないとき
if ( !File(reminsFile).exists() ) {
    println("Line:" + Throwable().stackTrace[0].lineNumber + " ★MainActivity★ ファイルが存在しない ★")

    //回顧録データのダミーファイルを作成する
    val text = "Dummy"
    try {
        val wrtRemStream = FileOutputStream(reminsFile)
        wrtRemStream.write(text.toByteArray()) //Dummyデータ
        wrtRemStream.close()
        println("Line:" + Throwable().stackTrace[0].lineNumber + " ★MainActivity★Dummyファイルを書き込んだ。★")
    } catch (e: IOException) {
        e.printStackTrace()
    }
}
} //makeDummyFile() */

} //MainActivity