package com.athena.reminiscence.db

import android.annotation.SuppressLint
import android.content.ContentValues
import android.content.Context
import android.database.sqlite.SQLiteDatabase
import android.database.sqlite.SQLiteOpenHelper
import android.os.Build
import androidx.annotation.RequiresApi
import com.athena.reminiscence.R
import com.athena.reminiscence.reminsFile
import com.athena.reminiscence.sharedPref
import java.io.File
import java.io.FileOutputStream
import java.io.IOException
import java.time.LocalDateTime.now
import java.time.format.DateTimeFormatter.ofPattern


//クラスを宣言する
class DatabaseHelper internal constructor(private val context: Context?) :
    SQLiteOpenHelper(context, DATABASE_NAME, null, DATABASE_VERSION) {

    private lateinit var rdb: SQLiteDatabase
    private lateinit var wdb: SQLiteDatabase

    private val minYear = context?.getString(R.string.minYear)?.toInt() //登録できるユーザーの最小の生年(明治維新よりも前は元号を特定でないため扱えない)
    private val offStem = 4
    private val offZodc = 4

    //干支データを分解して配列に入れる
    private val sb: Array<String?>? = context?.getString(R.string.sBases)?.split(",")?.toTypedArray() //十干
    private val jb: Array<String?>? = context?.getString(R.string.jBases)?.split(",")?.toTypedArray() //よみ
    private val zd: Array<String?>? = context?.getString(R.string.zodiac)?.split(",")?.toTypedArray() //十二支

    //データベースを定義する
    companion object {

        //データベース名を設定する
        private const val DATABASE_NAME = "Reminiscence.db"

        //バージョン情報を設定する
        private const val DATABASE_VERSION = 1
    }

    //テーブルを生成する
    @SuppressLint("Recycle")
    override fun onCreate(db: SQLiteDatabase) {
        //println("Line:"+ Throwable().stackTrace[0].lineNumber +" ★DatabaseHelper onCreate★db=$db this=$this")

        //データベースを生成する(テーブルが存在しないときだけクリエイトする)
        db.execSQL("CREATE TABLE IF NOT EXISTS howoldTable (id integer PRIMARY KEY, name TEXT, kana TEXT, upnm TEXT, dnnm TEXT, begn TEXT, term TEXT)")
        db.execSQL("CREATE TABLE IF NOT EXISTS annalsTable (id integer PRIMARY KEY, age Int, year TEXT, jpns TEXT, stems TEXT, jbase TEXT, zodiac TEXT, annals TEXT, events TEXT, codicil TEXT)")
    }
    override fun onUpgrade(db: SQLiteDatabase, oldVersion: Int, newVersion: Int) {}

    //テーブルの初期化を振り分ける
    @SuppressLint("Recycle")
    fun initDistributer(req:String) { //← dialog_Set_Annals.kt / dialog_Set_Howold.kt
        //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★DatabaseHelper★initDistributer★req=$req")

        //プリファレンスの設定値を読み取る
        val born = sharedPref.getString("born", "").toString() //誕生日
        val sort = sharedPref.getString("sort", "").toString() //並順モード seq/syl

        //初期化するとき
        when (req) {

            //じぶん年表を初期化するとき
            "initAnnals" -> { initializeAnnalsTable(born) } //born:誕生日

            //年齢リストを初期化するとき
            "initHowold" -> { initializeHowoldTable(sort) } //sort:並順モード seq/syl

            //表示モードを変えるとき
            "dialogPale" -> { return }

            //並順モードを変えるとき
            "dialogSort" -> { return }
        }
        //println("Line:"+ Throwable().stackTrace[0].lineNumber +" ★DatabaseHelper★ req=$req sort=$sort born=$born")
    } //initDistributer()



//== ↓↓↓ 年表テーブル ↓↓↓ ============================================================================

    //年表テーブルを初期化する
    private fun initializeAnnalsTable(begn:String) {  //begn:誕生日
        //println("Line:"+ Throwable().stackTrace[0].lineNumber +" ★DatabaseHelper★ initializeAnnalsTable★ begn=$begn")

        //既存の全レコードを削除する
        wdb = this.writableDatabase
        wdb.delete("annalsTable", "id >= 1", null)

        //引数の誕生日が空のとき
        if ( begn=="" ) {
            return
        }
        //引数の誕生日が空でないとき
        else{

            //変数を定義する
            val ix = begn.substring(0, 4).toInt() //ユーザーの生まれ年
            val maxYear = ix + 99 //規定の終了年(年表での末尾の年)

            //履歴を設定する
            var annals :String //年表
            val etch = context?.getString(R.string.etched) //履歴(誕生、学齢、長寿)
            val ec: Array<String> = etch?.split(",")!!.toTypedArray()
            //println("Line:"+ Throwable().stackTrace[0].lineNumber +" ★annalsTableInitialize★ ec[0]=${ec[0]} ec[1]=${ec[1]} ec[2]=${ec[2]} ec[3]=${ec[3]} ec[4]=${ec[4]} ec[5]=${ec[5]} ec[6]=${ec[6]} ec[7]=${ec[7]} ec[8]=${ec[8]} ec[9]=${ec[9]} ec[10]=${ec[10]} ec[11]=${ec[11]} ec[12]=${ec[12]} ec[13]=${ec[13]} ec[14]=${ec[14]} ec[15]=${ec[15]} ec[16]=${ec[16]} ec[17]=${ec[17]}")

            var events :String //時事
            val evnt = context?.getString(R.string.events) //時事(東京オリンピック,大阪万博)
            val ev: Array<String> = evnt?.split(",")!!.toTypedArray()

            //繰り返しの変数を設定する
            var j = offStem // 十干
            var k = offZodc // 十二支
            var ag = 0 //年齢
            var js :String //略号

            //明治維新から規定の終了年まで繰り返す(干支の開始を特定するため)
            for (i: Int in minYear!!..maxYear) {

                //誕生から99歳までのとき
                if (i >= ix) {

                    //和暦年と略号を設定する
                    when (i) {
                        in 1868..1911 -> { var jx = (i - 1867).toString(); if (jx.length==1) { jx="0$jx" }; js="M$jx" } //明治
                        in 1912..1925 -> { var jx = (i - 1911).toString(); if (jx.length==1) { jx="0$jx" }; js="T$jx" } //大正
                        in 1926..1988 -> { var jx = (i - 1925).toString(); if (jx.length==1) { jx="0$jx" }; js="S$jx" } //昭和
                        in 1989..2018 -> { var jx = (i - 1988).toString(); if (jx.length==1) { jx="0$jx" }; js="H$jx" } //平成
                        else                -> { var jx = (i - 2018).toString(); if (jx.length==1) { jx="0$jx" }; js="R$jx" } //令和
                    }
                    //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★i=$i js=$js ag=$ag j=$j sx=$sx ds=$ds k=$k zx=$zx dz=$dz★")

                    //履歴を記入する
                    annals = when (ag) {
                        0  -> ec[0]
                        in 7..15  -> ec[(ag - 6)] //学齢
                        20 -> ec[10] //成人式
                        60 -> ec[11] //還暦
                        70 -> ec[12] //古希
                        77 -> ec[13] //喜寿
                        80 -> ec[14] //傘寿
                        88 -> ec[15] //米寿
                        90 -> ec[16] //卒寿
                        99 -> "$begn|${ec[17]}" //生年月日|白寿
                        else -> ""
                    }

                    //時事を記入する
                    events = when (i) {
                        ix   -> ev[0] //ec[0].substring(2, 5) + ec[0].substring(2, 5) //仮の表示幅の設定
                        1964 -> ev[1] //東京オリンピック
                        1970 -> ev[2] //大阪万博
                        2021 -> ev[1] //東京オリンピック
                        else -> ""
                    }

                    //年ごとのデータをカラム配列に入れる(特記は空欄のまま)
                    val vl = contentAnnalsTable(ag, i.toString(), js, sb?.get(j).toString(), jb?.get(j).toString(), zd?.get(k).toString(), annals, events, "")

                    //レコードを追加する
                    wdb.insert("annalsTable", null, vl)

                    //年齢を加算する
                    ag += 1
                } //if

                //十干と十二支を循環させる
                j += 1; if ( j==10 ) { j = 0 }
                k += 1; if ( k==12 ) { k = 0 }
            } //for

//********** ↓↓↓ オリジナルの年表を追加する ↓↓↓ **********

//********** ↑↑↑ オリジナルの年表を追加する ↑↑↑ **********

        } //if(begn!="")
        return
    } //initializeAnnalsTable

    //年表データを読み込む
    @SuppressLint("Recycle")
    fun readAnnalsData(): String {

        rdb = this.readableDatabase

        //年表データ読み取りのSQLを実行する
        val sql = "SELECT * FROM annalsTable"
        val cursor = rdb.rawQuery(sql, null)
        val count = cursor.count
        val num = cursor.columnCount
        //println("Line:"+ Throwable().stackTrace[0].lineNumber +" ★DatabaseHelper★readAnnalsData count=$count num=$num")

        //カラム数の配列変数を準備する
        val col = Array(count) { arrayOfNulls<String>(num) } //num=10
        var itmString = ""

        //読み取ったデータを分解して文字列に変換する
        cursor.moveToFirst()
        for (i in 0 until count) {

            //カラムごとにデータを取り出す
            col[i][0] = cursor.getInt(0).toString() //id
            col[i][1] = cursor.getInt(1).toString() //age
            col[i][2] = cursor.getString(2) //year
            col[i][3] = cursor.getString(3) //jpns
            col[i][4] = cursor.getString(4) //stems
            col[i][5] = cursor.getString(5) //jbase
            col[i][6] = cursor.getString(6) //zodiac
            col[i][7] = cursor.getString(7) //annals
            col[i][8] = cursor.getString(8) //events
            col[i][9] = cursor.getString(9) //codicil
            itmString += col[i][0] + "\t" + col[i][1] + "\t" + col[i][2] + "\t" + col[i][3] + "\t" + col[i][4] + "\t" + col[i][5] + "\t" + col[i][6] + "\t" + col[i][7] + "\t" + col[i][8] + "\t" + col[i][9] + "\n"

            //println("Line:"+ Throwable().stackTrace[0].lineNumber +" ★DatabaseHelper★readAnnalsData itmString=$itmString")
            //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★DatabaseHelper★readAnnalsData i=$i col[i][0]=${col[i][0]} col[i][1]=${col[i][1]} col[i][2]=${col[i][2]} col[i][3]=${col[i][3]} col[i][4]=${col[i][4]} col[i][5]=${col[i][5]} col[i][6]=${col[i][6]} col[i][7]=${col[i][7]} col[i][8]=${col[i][8]} col[i][9]=${col[i][9]} col[i][7].length=${col[i][7].toString().length} col[i][8].length=${col[i][8].toString().length}")
            cursor.moveToNext()
        }
        cursor.close()
      //db.close()

        //エミュレーターのゴミを排除する(cursorに古いデータが残っている)
        /*if ( col[99][7]=="白寿" ){
            println("Line:" + Throwable().stackTrace[0].lineNumber + " ★DatabaseHelper★readAnnalsData col[99][7]=${col[99][7]}")
            itmString = "\n" //クリアしてDummyを入れる
        }*/

        //println("Line:"+ Throwable().stackTrace[0].lineNumber +" ★DatabaseHelper★readAnnalsData itmString=\n$itmString")
        return itmString.dropLast(1)
    } //readAnnalsData

    //年表データを更新する
    fun updateAnnalsData(age:Int, year:String, jpns:String, stems:String, jbase:String, zodiac:String, annals:String, events:String, codicil:String): Boolean {
        //println("Line:"+ Throwable().stackTrace[0].lineNumber +"★DatabaseHelper★\n age=$age \n annals=$annals \n events=$events \n codicil=$codicil")

        //行番号(年齢)からレコードidを設定する
        val id = age + 1

        //更新データをカラム配列に入れる
        val vl = contentAnnalsTable(age, year, jpns, stems, jbase, zodiac, annals, events, codicil)
        val whereClauses = "id = $id"
        val whereArgs = null //arrayOf(id.toString(),)
        //println("Line:"+ Throwable().stackTrace[0].lineNumber +" ★DatabaseHelper★ id=$id whereArgs=$whereArgs ")

        //レコードを更新する
        wdb = this.writableDatabase
        wdb.update("annalsTable", vl, whereClauses, whereArgs)

        return true
    }

    //【関数】年表データをカラム配列に入れる
    private fun contentAnnalsTable(age:Int, year:String, jpns:String, stems:String, jbase:String, zodiac:String, annals:String, events:String, codicil:String): ContentValues {
        //println("Line:"+ Throwable().stackTrace[0].lineNumber +" ★DatabaseHelper★contentAnnalsTable annals=$annals eventss=$events codicil=$codicil")

        val vl = ContentValues().apply {
            put("age", age)
            put("year", year)
            put("jpns", jpns)
            put("stems", stems)
            put("jbase", jbase)
            put("zodiac", zodiac)
            put("annals", annals)
            put("events", events)
            put("codicil", codicil)
        }
        return vl
    }

//== ↓↓↓ 年齢リスト ↓↓↓ ==============================================================================

    //年齢テーブルを初期化する(ユーザー定数は含まない)
    private fun initializeHowoldTable(attach:String){
        //println("Line:"+ Throwable().stackTrace[0].lineNumber +"★DatabaseHelper★initializeHowoldTable attach=$attach★")

        //ユーザー定数を除く全レコードを削除する
        wdb = this.writableDatabase
        wdb.delete("howoldTable", "id >= 1", null)

        //サンプルデータを付けるとき
        if ( attach=="seq" ) {

            //任意のデータを準備する(ユーザー定数は含めない)
            val iData = arrayOf(

                arrayOf("吉田 茂", "よしだ しげる", "西暦", "西暦", "1878/09/22", "1967/10/20"),
                arrayOf("野田 〇〇", "のだ 〇〇〇〇", "昭和", "", "1957/05/20", ""),
                arrayOf("安倍 晋三", "あべ しんぞう", "昭和", "令和", "1944/12/12", "2022/07/08"),
                arrayOf("結婚記念日", "ん_1", "西暦", "西暦", "1983/06/15", "")

            )
            /*
            println("Line:"+ Throwable().stackTrace[0].lineNumber +"★DatabaseHelper★ count = ${iData.count()}")
            for (i in 0 until iData.count()) {
                println("Line:" + Throwable().stackTrace[0].lineNumber + " ${iData[i][0]},${iData[i][1]},${iData[i][2]},${iData[i][3]},${iData[i][4]},${iData[i][5]}")
            }
            */

            //初期データをデータベースに書き込む
            for (i in 0 until iData.count()) {
                //println("Line:"+ Throwable().stackTrace[0].lineNumber +"★DatabaseHelper★ iData[0]=${iData[0]} iData[0]=${iData[0]}")

                //データをカラム配列に入れる
                val vl = contentHowoldTable(iData[i][0], iData[i][1], iData[i][2], iData[i][3], iData[i][4], iData[i][5])
                //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★vl=$vl★")

                //レコードを追加する
                wdb.insert("howoldTable", null, vl)
            }
            //println("Line:"+ Throwable().stackTrace[0].lineNumber +" ★initializeHowoldTable")
        } //with sample
        return
    } //initializeHowoldTable

    //年齢データを挿入する
    fun insertHowoldData(name:String, kana:String, upnm:String, dnnm:String, begn:String, term:String): Boolean {
        //println("Line:"+ Throwable().stackTrace[0].lineNumber +"★DatabaseHelper★\n name=$name \n kana=$kana \n upnm=$upnm \n dnnm=$dnnm \n begn=$begn \n term=$term")

        //追加データをdb配列に入れる
        val vl = contentHowoldTable(name, kana, upnm, dnnm, begn, term)
        //println("Line:"+ Throwable().stackTrace[0].lineNumber +"★DatabaseHelper★ vl=$vl")

        //レコードを追加する
        wdb = this.writableDatabase
        wdb.insert("howoldTable", null, vl)

        return true
    }

    //年齢データを更新する
    fun updateHowoldData(id:Int, name:String, kana:String, upnm:String, dnnm:String, begn:String, term:String): Boolean {
        //println("Line:"+ Throwable().stackTrace[0].lineNumber +"★DatabaseHelper★ id=$id name=$name kana=$kana upnm=$upnm dnnm=$dnnm begn=$begn term=$term")

        //更新データをdb配列に入れる
        val vl = contentHowoldTable(name, kana, upnm, dnnm, begn, term)
        val whereClauses = "id = $id"
        val whereArgs = null //arrayOf(id.toString())
        //println("Line:"+ Throwable().stackTrace[0].lineNumber +" ★DatabaseHelper★ id=$id whereArgs=$whereArgs vl:$vl")

        //レコードを更新する
        wdb = this.writableDatabase
        wdb.update("howoldTable", vl, whereClauses, whereArgs)

        return true
    }

    //【関数】年齢データをカラム配列に入れる
    private fun contentHowoldTable(name:String, kana:String, upnm:String, dnnm:String, begn:String, term:String): ContentValues {
        val vl = ContentValues().apply {
            put("name", name)
            put("kana", kana)
            put("upnm", upnm)
            put("dnnm", dnnm)
            put("begn", begn)
            put("term", term)
        }
        return vl
    }

    //年齢データを削除する
    fun deleteHowoldData(id:Int): Boolean {
        //println("Line:"+ Throwable().stackTrace[0].lineNumber +"★DatabaseHelper deleteHowoldData★ id=$id)

        //レコードを削除する
        wdb = this.writableDatabase
        wdb.delete("howoldTable", "id = $id", null)

        return true
    }

    //年齢データを読み込む
    @SuppressLint("Recycle")
    fun readHowoldData(dnnm:String): Triple<Int, Int, String> {
        //println("Line:"+ Throwable().stackTrace[0].lineNumber +" ★DatabaseHelper readHowoldData dnnm=$dnnm ")

        rdb = this.readableDatabase

        //戻り値の変数を準備する
        var pcnt = 0 //0:past 過去の件数
        var lcnt = 0 //1:live 現在の件数
        var itmString = ""

        //年齢データ読み取りのSQLを実行する
        var sql = "SELECT * FROM howoldTable"
        if ( dnnm=="syl" ) {
            sql += " ORDER BY kana ASC"
        }
        val cursor = rdb.rawQuery(sql, null)
        val count = cursor.count
        val num = cursor.columnCount
        //println("Line:"+ Throwable().stackTrace[0].lineNumber +" ★DatabaseHelper★readHowoldData count=$count num=$num ")

        //読み取ったデータを分解して文字列に変換する
        val curDate = now().format(ofPattern("yyyy/MM/dd"))
        val col = Array(cursor.count) { arrayOfNulls<String>(num) }
        cursor.moveToFirst()
        for (i in 0 until count) {

            //カラムごとにデータを取り出す
            col[i][0] = cursor.getInt(0).toString() //rcid
            col[i][1] = cursor.getString(1) //name
            col[i][2] = cursor.getString(2) //upnm
            col[i][3] = cursor.getString(3) //kana
            col[i][4] = cursor.getString(4) //dnnm
            col[i][5] = cursor.getString(5) //begn
            col[i][6] = cursor.getString(6) //term
            itmString += col[i][0] +","+ col[i][1] +","+ col[i][2] +","+ col[i][3] +","+ col[i][4] +","+ col[i][5] +","+ col[i][6] +"\n"
            //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★DatabaseHelper itmString=$itmString")

            //[終了(満期)]の日付が入っていない件数をかぞえる(現在)
            if ( col[i][6]=="" || col[i][6]!! > curDate ) {
                lcnt += 1
                //println("Line:"+ Throwable().stackTrace[0].lineNumber +" DatabaseHelper lcnt=$lcnt col[i][3]="+ col[i][3] )
            }

            //[終了(満期)]の日付が入っている件数をかぞえる(過去)
            if ( col[i][6]!="" && col[i][6]!! <= curDate ) {
                pcnt += 1
                //println("Line:"+ Throwable().stackTrace[0].lineNumber +" DatabaseHelper pcnt=$pcnt col[i][3]="+ col[i][3] )
            }

            cursor.moveToNext()
        } //for
        cursor.close()
      //db.close()
        itmString = itmString.dropLast(1)
        //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★DatabaseHelper itmString=$itmString")

        //戻り値を返す
        return Triple(pcnt, lcnt, itmString)
    } //readHowoldData

    //【関数】生まれ年から干支を取得する
    fun getZodiac(begnNumber:String):String {
        //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★DatabaseHelper getZodiac★ begnNumber=$begnNumber")

        //変数を定義する
        val bYear = begnNumber.substring(0, 4).toInt() //生まれ年
        val cYear = now().format(ofPattern("yyyy/MM/dd")).substring(0,4).toInt()
        var sjz = "" //取得した干支
        //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★DatabaseHelper getZodiac★ minYear=$minYear cYear=$cYear begnNumber=$begnNumber bYear=$bYear")

        //干支のずれを補正する()
        var j = offStem // 十干
        var k = offZodc // 十二支

        //規定の範囲内(minYear～cYear)を繰り返す
        for (i: Int in minYear!!..cYear) {
            //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★DatabaseHelper getZodiac★ minYear=$minYear cYear=$cYear i=$i bYear=$bYear")

            //生まれ年と一致したとき
            if (i == bYear) {
                //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★DatabaseHelper getZodiac i=$i bYear=$bYear★")

                //干支を配列から読み取る
                sjz = sb?.get(j).toString() + "/" + jb?.get(j).toString() + "/" + zd?.get(k).toString()
                break
            }

            //十干と十二支を循環させる
            j += 1; if (j == 10) { j = 0 }
            k += 1; if (k == 12) { k = 0 }
        } //for

        val (stem, ruby, zodc) = sjz.split("/")
        //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★DatabaseHelper getZodiac stem=$stem ruby=$ruby zodc=$zodc★")

        //取得した干支を表示しやすい形で返す
        return "$stem($ruby)/$zodc"
    }


//== ↓↓↓ Backup & Restore ↓↓↓ ==============================================================================

    //[回顧録データを退避する]とき
    @RequiresApi(Build.VERSION_CODES.TIRAMISU)
    @SuppressLint("Recycle", "SdCardPath", "SuspiciousIndentation")
    fun saveReminsData(): Boolean {
        //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★DatabaseHelper★saveReminsData★ 回顧録データを退避する")

        //データベースから回顧録データを読み込む
        val (_, _, howoldData) = readHowoldData("seq") //pcnt, lcnt, howoldData
        val annalsData = readAnnalsData()+ "\n"
        var chkWrt = true
        //println("Line:"+ Throwable().stackTrace[0].lineNumber +" ★DatabaseHelper★saveReminsData★\n★reminsFile=$reminsFile\n★annalsData=\n${annalsData.substring(0,100)}\n★howoldData=\n$howoldData\n\n")

        //回顧録データを退避ファイルに書き込む
        try {
            val wrtRemStream = FileOutputStream(File(reminsFile), true)
            wrtRemStream.write(annalsData.toByteArray()) //年表データ
            wrtRemStream.write(howoldData.toByteArray()) //年齢リスト
            wrtRemStream.close()
            //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★DatabaseHelper★ reminsFileファイルを書き込んだ。★")

        } catch (e: IOException) {
            e.printStackTrace()
            chkWrt = false
            //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★DatabaseHelper★ reminsFileファイルの書き込みエラー!!★")
        }

        return chkWrt //true or false

    } //saveReminsData


    //[回顧録データを復元する]
    @RequiresApi(Build.VERSION_CODES.R)
    fun restoreReminsData(lines: List<String>): Boolean {
        //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★DatabaseHelper★restoreReminsData★[回顧録データを復元する]★\n★lines=$lines")

        //退避ファイルのデータを読み取れたとき
        var rst = false
        if( lines.isNotEmpty() ) {
            //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★DatabaseHelper★lines=$lines")

            //データペースの回顧録データをテーブルごと削除してから再生成する
            wdb = this.writableDatabase
            wdb.execSQL("DROP TABLE if exists howoldTable")
            wdb.execSQL("DROP TABLE if exists annalsTable")
            wdb.execSQL("CREATE TABLE IF NOT EXISTS howoldTable (id integer PRIMARY KEY, name TEXT, kana TEXT, upnm TEXT, dnnm TEXT, begn TEXT, term TEXT)")
            wdb.execSQL("CREATE TABLE IF NOT EXISTS annalsTable (id integer PRIMARY KEY, age Int, year TEXT, jpns TEXT, stems TEXT, jbase TEXT, zodiac TEXT, annals TEXT, events TEXT, codicil TEXT)")

            //ファイルの末尾まで繰り返す
            for (line in lines) {

                //年表年齢テーブルを復元する
                restoreTables(line)
            }
            rst = true
        }

        return rst
    } //restoreReminsData

    //年表年齢テーブルを復元する
    private fun restoreTables(lineData:String){

        //行ごとに列数を読み取る (じぶん年表と年齢リストの振り分けに使う)
        val size = lineData.split("\t").count()
        //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★ size=$size lineDatat=$lineData")

        //年表テーブルに書き込む
        if ( size == 10 ) { //annalsData

            //行データを分解する
            val fld: List<String> = lineData.split("\t")

            //ユーザー定数の誕生日を書き替える (機種変更による復元のため)
            if ( fld[1]=="99" ) {
                //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★DatabaseHelper★restoreReminsData fld[0]=${fld[0]} fld[1]=${fld[1]} fld[7]=${fld[7]} born=${fld[7].substring(0, 10)}")
                sharedPref.edit().putString("born", fld[7].substring(0, 10)).apply()
            }

            //行データを配列に入れる
            val vl =
                contentAnnalsTable( fld[1].toInt(), fld[2], fld[3], fld[4], fld[5], fld[6], fld[7], fld[8], fld[9] )
            //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★DatabaseHelper★restoreReminsData vl=$vl")


            //年表データをデータベースに書き込む
            wdb.insert("annalsTable", null, vl)
        } //size=10 annalsData

        //年齢テーブルに書き込む
        else { //size=1 howoldData

            //行データを分解する
            val how = lineData.split(",")
            //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★DatabaseHelper★restoreReminsData★ how[0]=${how[0]} how[1]=${how[1]} how[2]=${how[2]} how[3]=${how[3]} how[4]=${how[4]} how[5]=${how[5]} how[6]=${how[6]}")

            //行データを配列に入れる
            val vl = contentHowoldTable(how[1], how[2], how[3], how[4], how[5], how[6])
            //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★DatabaseHelper★restoreReminsData★ vl=$vl ")

            //年齢データをデータベースに書き込む
            wdb.insert("howoldTable", null, vl)
        } //size=0 howoldData
    } //restoreTables

//== ↓↓↓ 再起動 ↓↓↓ (記録のために消さない) ==============================================================
    //restartMain("DspAnnals") 遷移先を引数として再起動の関数を呼ぶ

    /*//MainActivityを再起動する
    private val mainConstract: Context? = context
    private fun restartMain(destination:String) {
        //println("Line:"+ Throwable().stackTrace[0].lineNumber +" ★restartMain★ destination=$destination")

        //引数をセットする
        val intent = Intent(mainConstract, MainActivity::class.java)
        intent.putExtra("argTEXT", destination)
        mainConstract?.startActivity(intent)
    }*/

} //class DatabaseHelper

