package com.athena.reminiscence.db

import android.annotation.SuppressLint
import com.athena.reminiscence.reminsFile
import java.io.File
import java.nio.file.Files
import java.nio.file.Paths
import java.nio.file.attribute.FileTime
import java.text.SimpleDateFormat
import java.time.LocalDate
import java.time.OffsetDateTime
import java.time.Period
import java.time.ZoneOffset
import java.time.format.DateTimeFormatter
import java.util.*


//クラスを宣言する
class DateUtility {

    //当日日付を取得する
    @SuppressLint("SimpleDateFormat")
    val curDate : String = SimpleDateFormat("yyyy/MM/dd").format(Date())

    //当日のタイムスタンプを生成する (ファイル名に当日の日付を持たせるときに使う)
    /*fun getDateTimeString(): String {
        val curDateTime = LocalDateTime.now()
        //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★dateTimeDateTime★ curDate=$curDate curDateTime=$curDateTime")
        val yy = curDateTime.toString().substring(0, 4)
        val mm = curDateTime.toString().substring(5, 7)
        val dd = curDateTime.toString().substring(8, 10)
        val hh = curDateTime.toString().substring(11, 13)
        val nn = curDateTime.toString().substring(14, 16)
        val ss = curDateTime.toString().substring(17, 19)
        //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★dateTimeDateTime★ yy=$yy mm=$mm dd=$dd hh=$hh nn=$nn ss=$ss")
        //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★dateTimeDateTime★ $yy$mm$dd" + "_$hh$nn$ss")
        return "$yy$mm$dd"+"_$hh$nn$ss"
    }*/

    //退避ファイルのタイムスタンプを取得する
    fun getTimeStamp(): Pair<String, String> {
        //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★DateUtility★getTimeStamp()★\n★reminsFile=$reminsFile")

        //退避ファイルが存在するとき
        var stamp = ""
        var size = ""
        if ( File(reminsFile).exists() ) {
            //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★DateUtility★File exists!!")

            size = File(reminsFile).length().toString()
            //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★DateUtility★size=$size")

            //ファイル名の日付からタイムスタンプを取得する
            val timeStamp: FileTime = Files.getLastModifiedTime(Paths.get(reminsFile))
            //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★DateUtility★timeStamp=$timeStamp")

            val datetime = OffsetDateTime.parse(timeStamp.toString())
            val lt = datetime.withOffsetSameInstant(ZoneOffset.ofHours(9))
            val e = lt.toString().substring(0, 10).replace("-", "/")
            val t = lt.toString().substring(11, 19)
            //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★DateUtility★lt=$lt ★e=$e ★t=$t")

            stamp = "$e $t" //2024/04/15 07:59:00

        }
        return Pair(stamp, size)
    }

    //【関数】年(yy)が正しいことをチェックする
    @SuppressLint("NewApi")
    fun checkYY(idx:Int, minBegn: Int?, jy:String, yy:String): Boolean {
        var ychk = false; if ( idx==0 || ( jy=="" && yy=="" ) ) { return false }
        //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★dateCheck★ curDate=$curDate idx=$idx minBegn=$minBegn jy=$jy yy=$yy ")

        val maxTerm = curDate.substring(0,4).toInt() + 99
        var y = 0
        if ( jy=="" ) { y = yy.toInt() } //和暦が空のときは西暦年を基準にする
        if ( yy=="" ) { y = jy.toInt() } //西暦が空のときは和暦年を基準にする
        var jYY = 0 //元号ごとの和暦年
        //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★dateCheck★ idx=$idx minBegn=$minBegn maxTerm=$maxTerm jy=$jy yy=$yy y=$y ")

        //西暦のとき(範囲内か)
        if ( idx == 1 ) {
            //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★dateCheck★ idx=$idx yy=$yy")

            val yn = yy.toInt()
            if (yn>= minBegn!! && yn<=maxTerm) {
                ychk = true
            }
            return ychk
        }

        //和暦のとき(元号に応じて正しいか)
        else if( idx>=2 ) {
            //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★dateCheck★ idx=$idx yy=$yy")

            when(idx){
                2 -> { if (y in 1..45) { jYY = y } } //明治
                3 -> { if (y in 1..15) { jYY = y } } //大正
                4 -> { if (y in 1..64) { jYY = y } } //昭和
                5 -> { if (y in 1..31) { jYY = y } } //平成
                6 -> { if (y in 1..99) { jYY = y } } //令和
                else -> { jYY = 0 }
            }
        }
        if ( jYY!=0 ) { ychk = true }
        //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★dateCheck★ idx=$idx yy=$yy jYY=$jYY ychk=$ychk")

        return ychk
    } //checkYY

    //【関数】月(mm)が正しいことをチェックする
    fun checkMM(mm:String): Boolean {
        //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★checkMM★ mm=$mm")
        var mchk = false
        val mn = mm.toInt()
        if (mn in 1..12) { //mn>=1 && mn<=12

            mchk = true
        }
        //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★checkMM★ mm=$mm mchk=$mchk")
        return mchk
    }

    //【関数】日付(dd)の妥当性をチェックする
    fun checkDD(mm:String, dd:String):Boolean {
        //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★checkDD★ mm=$mm dd=$dd")
        var dchk = true

        if ( checkMM(mm) ) {
            //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★checkDD★ mm=$mm dd=$dd l=${dd.length} ★")

            //肯定形は使える   dd=="" dd.isEmpty()    dd.isBlank()
            //否定形は使えない dd!="" dd.isNotEmpty() dd.isNotBlank()
            if ( dd.isNotEmpty() ) {
                //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★checkDD★ mm=$mm dd=$dd l=${dd.length} ★")
                val smlArray: IntArray = intArrayOf(2, 4, 6, 9, 11)
                val bigArray: IntArray = intArrayOf(1, 3, 5, 7, 8, 10, 12)
                val m = mm.toInt()
                val d = dd.toInt()
                val sml = m in smlArray
                val big = m in bigArray
                //dchk = true
                //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★checkDD★ mm=$mm dd=$dd l=${dd.length} m=$m d=$d sml=$sml big=$big ★")

                //大の月のとき
                if ( big ) { //big==true
                    //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★checkDD★ big=$big m=$m d=$d ★")
                    if (d < 1 || d > 31) {
                        dchk = false
                    }
                }

                //小の月のとき
                if ( sml ) { //sml==true
                    //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★checkDD★sml=$sml m=$m d=$d ★")
                    if (m == 2) {
                        if (d < 1 || d > 29) {
                            dchk = false
                        }
                    } else {
                        if (d < 1 || d > 30) {
                            dchk = false
                        }
                    }
                }
            }
        }
        //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★checkDD★ mm=$mm dd=$dd dchk=$dchk ★")
        return dchk
    } //checkDD

    //【関数】日付の妥当性を確認する
    @SuppressLint("SuspiciousIndentation")
    fun checkYMD(ymd:String, minYear:Int): Boolean {
      //ymd: 開始日または終了日
      //minYear: 明治維新
        var rs = false
        val maxTerm = curDate.substring(0,4).toInt() + 99
        //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★checkYMD★ymd=$ymd")

        //年(yy)が1868年以降でありかつ当日から99年後までのとき
        if (ymd.substring(0,4).toInt() in minYear..maxTerm) {
            //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★checkYMD★ymd=$ymd★")

            //月(mm)が1月から12月のとき
            val arrayYMD = ymd.split("/")
            val cy = arrayYMD[0]
            var cm = arrayYMD[1]; if( cm.length==1 ){ cm = "0$cm" }
            var cd = arrayYMD[2]; if( cd.length==1 ){ cd = "0$cd" }
            //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★checkYMD★ymd=$ymd arrayYMD=$arrayYMD cy=$cy cm=$cm cd=$cd★")

            if (cm.toInt() in 1..12) {

                //日(dd)が妥当なとき
                if ( checkDD(cm, cd) ) {
                    val dtf: DateTimeFormatter = DateTimeFormatter.ofPattern("yyyy/MM/dd")
                    dtf.format(LocalDate.parse("$cy/$cm/$cd", dtf))
                    rs = true
                }
            }
        }
        return rs
    }

    //【関数】西暦年を和暦年に変換する
    fun changeAnnoToJpn(yy:String, mm:String, dd:String, iEra:Array<String>): Array<String> {
        //yy 西暦年
        //mm 月
        //dd 日
        //iEra 元号
        //println("Line:" + Throwable().stackTrace[0].lineNumber +" ★changeAnnoToJpn★yy=$yy mm=$mm dd=$dd iEra=$iEra ★")

        //初期値を設定する
        val gx :String //jg 元号
        val jx :String //jy 和暦年

        //西暦年から年月ごとの和暦年と元号を特定する
        val yn = yy.toInt()
        val mn = mm.toInt()
        val dn = dd.toInt()
        if      ((yn == 2019) && (mn < 5))               { gx = iEra[5]; jx = "31" } //平成
        else if ((yn == 1989) && (mn <  2) && (dn <  8)) { gx = iEra[4]; jx = "64" } //昭和
        else if ((yn == 1926) && (mn < 13) && (dn < 26)) { gx = iEra[3]; jx = "15" } //大正
        else if ((yn == 1926) && (mn < 12))              { gx = iEra[3]; jx = "15" } //大正
        else if ((yn == 1868) && (mn <  8) && (dn < 31)) { gx = iEra[2]; jx = "45" } //明治
        else if ((yn == 1868) && (mn <  7))              { gx = iEra[2]; jx = "45" } //明治
        else if ( yn >  2018) { gx = iEra[6]; jx = (yn - 2018).toString() } //令和
        else if ( yn >  1988) { gx = iEra[5]; jx = (yn - 1988).toString() } //平成
        else if ( yn >  1925) { gx = iEra[4]; jx = (yn - 1925).toString() } //昭和
        else if ( yn >  1911) { gx = iEra[3]; jx = (yn - 1911).toString() } //大正
        else                  { gx = iEra[2]; jx = (yn - 1867).toString() } //明治
        //println("Line:" + Throwable().stackTrace[0].lineNumber +" ★changeAnnoToJpn★ gx=$gx jx=$jx★")

        //元号から略号を特定する
        val abbr = changeJpnToAbbr(gx, iEra)
        //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★changeAnnoToJpn★ gx=$gx jx=$jx abbr=$abbr ★")

        return arrayOf(gx, jx, abbr) //元号,和暦年,略号

    } //changeAnnoToJpn

    //【関数】和暦を西暦に変換する
    fun changeJpnToAnno(jg:String, jx:String, mx:String, dx:String, iEra:Array<String>): Array<String> {
        //jg 元号
        //jx 和暦年
        //mx 月
        //dx 日
        //iEra
        //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★changeAnnoToJpn★ jg=$jg jx=$jx iEra=$iEra ★")

        var yn = "" //西暦年
        when (jg) {
            iEra[2] -> { yn = (jx.toInt() + 1867).toString() }
            iEra[3] -> { yn = (jx.toInt() + 1911).toString() }
            iEra[4] -> { yn = (jx.toInt() + 1925).toString() }
            iEra[5] -> { yn = (jx.toInt() + 1988).toString() }
            iEra[6] -> { yn = (jx.toInt() + 2018).toString() }
        }
        //略号と月日を特定する
        val ax = changeJpnToAbbr(jg, iEra)
        var mm = mx; if( mm.length==1 ){ mm = "0$mm" }
        var dd = dx; if( dd.length==1 ){ dd = "0$dd" }
        //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★changeAnnoToJpn★ yn=$yn ax=$ax mm=$mm dd=$dd ★")

        return arrayOf(yn, ax, mm, dd) //西暦年, 和暦略号, 月, 日
    }

    //元号から略号を特定する
    private fun changeJpnToAbbr(jg:String, iEra:Array<String>): String {
        var abbr = ""
        when (jg) {
            iEra[1] -> { abbr = "A" }
            iEra[2] -> { abbr = "M" }
            iEra[3] -> { abbr = "T" }
            iEra[4] -> { abbr = "S" }
            iEra[5] -> { abbr = "H" }
            iEra[6] -> { abbr = "R" }
        }
        //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★changeAnnoToJpn★ abbr=$abbr jg=$jg★")
        return abbr
    }

    //【関数】年齢を計算する
    fun calcAge(itmName: String, itmBegn: String, itmTerm: String): String {
        //itmName 名前(スペースの有無により人か時事かを判定する)
        //itmBegn 開始日
        //itmTerm 終了日

        //開始日が空のとき(年齢を0.0として返す)
        if ( itmBegn=="" ) { return "0.0" }

        //開始日を分解する
        val bgn = itmBegn.split("/")
        val bgnYY = bgn[0]
        val bgnMM = bgn[1]
        val bgnDD = bgn[2]
        val fuz: Array<String>
        //println("Line:"+ Throwable().stackTrace[0].lineNumber +" ★ageCalculation★ itmBegn=$itmBegn bgnYY=$bgnYY bgnMM=$bgnMM bgnDD=$bgnDD")

        //終了日が入っていないとき(当日日付を取得して仮の終了日する)
        if ( itmTerm=="" ) {
            fuz = curDate.split("/").toTypedArray()
        }

        //終了日が入っているとき(期間or年齢を判定する)
        else {

            //名前の空白を取り除く(長さの変化を調べる)
            val tmpName = itmName.replace("\\s".toRegex(), "")

            //名前に空白がなかったとき(名称なので期間を対象とする)
            if ( itmName.length==tmpName.length ) {
                //println("Line:"+ Throwable().stackTrace[0].lineNumber +" ★ageCalculation★ itmName=$itmName itmBegn=$itmBegn itmTerm=$itmTerm")

                //終了日を翌日の日付とする(終了日の前日を満期日と計算させる)
                val formatter = DateTimeFormatter.ofPattern("yyyy/MM/dd")
                val date = LocalDate.parse(itmTerm, formatter)
                val period = Period.of(0, 0, 1)
                val nxtDate = date.plus(period).toString()
                fuz = nxtDate.split("-").toTypedArray()
                //println("Line:"+ Throwable().stackTrace[0].lineNumber +" ★ageCalculation★ date=$date nxtDate=$nxtDate fuz[0]=$fuz[0] fuz[1]=$fuz[1] fuz[2]=$fuz[2]")
            }

            //名前に空白があったとき(氏名なので年齢を対象とする)
            else {

                //終了日を取得する
                fuz = itmTerm.split("/").toTypedArray()
            }
        }

        //計算のための終了日を設定する
        val fuzYY = fuz[0]
        val fuzMM = fuz[1]
        val fuzDD = fuz[2]
        //println("Line:"+ Throwable().stackTrace[0].lineNumber +" ★ageCalculation★ fuzYY=$fuzYY fuzMM=$fuzMM fuzDD=$fuzDD")

        //[現在年]-[生まれ年]で年齢を計算する
        var years = fuzYY.toInt() - bgnYY.toInt()
        var month = 0
        //println("Line:"+ Throwable().stackTrace[0].lineNumber +" ★ageCalculation★ years=$years month=$month")

        //誕生月を迎えていなければ年齢-1にする
        if (bgnMM > fuzMM) {
            years -= 1
            month = 12 - (bgnMM.toInt() - fuzMM.toInt())

            //誕生月が12月のとき
            if( bgnMM=="12" ){

                //誕生日の前のとき
                if(bgnDD > fuzDD ){
                    month -= 1
                }
            }
        }

        //誕生月のとき
        else if (bgnMM == fuzMM) {

            //誕生日前のとき
            if ( bgnDD > fuzDD) {
                month = 12 - (bgnMM.toInt() - fuzMM.toInt()) - 1
                years -= 1
            }

            //誕生日以降のとき
            if( fuzDD >= bgnDD ){
                month = 0
            }
        }

        //誕生月を過ぎているとき
        if (fuzMM > bgnMM) {

            //誕生日前のとき
            month = fuzMM.toInt() - bgnMM.toInt()
        }
        //println("Line:"+ Throwable().stackTrace[0].lineNumber +" ★ageCalculation★ years=$years month=$month")

        //計算結果を戻す
        return "$years.$month"
    } //calcAge
} //class