package com.athena.reminiscence.ui.dialog

import android.annotation.SuppressLint
import android.app.Dialog
import android.content.Context
import android.content.res.Configuration
import android.graphics.Color
import android.os.Build
import android.os.Bundle
import android.text.Editable
import android.text.TextWatcher
import android.view.View
import android.view.inputmethod.EditorInfo
import android.view.inputmethod.InputMethodManager
import android.widget.Button
import android.widget.EditText
import android.widget.RadioButton
import android.widget.TextView
import android.widget.Toast
import androidx.activity.result.contract.ActivityResultContracts
import androidx.annotation.RequiresApi
import androidx.fragment.app.DialogFragment
import com.athena.reminiscence.R
import com.athena.reminiscence.db.DatabaseHelper
import com.athena.reminiscence.db.DateUtility
import com.athena.reminiscence.annalsData
import com.athena.reminiscence.reminsFile
import com.athena.reminiscence.sharedPref
import com.athena.reminiscence.ui.main.FragmentDspAnnals
import java.io.File
import java.io.FileOutputStream
import java.io.IOException


//クラスを宣言する
class SetAnnalsDialog : DialogFragment() {

    //共用変数を定義する
    private lateinit var db: DatabaseHelper
    private val du = DateUtility()
    private var born = ""
    private var tmod = ""
    private var done = ""

    //ダイアログを生成する
    @RequiresApi(Build.VERSION_CODES.TIRAMISU)
    @SuppressLint("CutPasteId", "SuspiciousIndentation")
    override fun onCreateDialog(savedInstanceState: Bundle?): Dialog {

//==================================================================================================
        //SAFを使ってファイルを選択する(読取りの許可を得る)(api33以上のみ)
        val getContentLauncher = registerForActivityResult( // ← Line 487 buttonOkok.setOnClickListener{}
            ActivityResultContracts.GetContent()
        ) { _ -> //uri
            //println("Line:"+ Throwable().stackTrace[0].lineNumber +" ★SetAnnalsDialog★ 退避ファイルを選択した ★" )
            //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★MainActivity★\n★rms=$reminsFile\n★uri=$uri\n★authority=${uri?.authority!!}\n★host=${uri.host!!}\n★lastPathSegment=${uri.lastPathSegment!!}\n★path=${uri.path!!}\n★scheme=${uri.scheme!!}/n★schemeSP=${uri.schemeSpecificPart}")

            //【asus-1006D】【oppo-cph2603】
            //★rms=/storage/emulated/0/Documents/Remins.txt
            //★uri=content://com.android.externalstorage.documents/document/primary%3ADocuments%2FRemins.txt
            //★authority=com.android.externalstorage.documents
            //★host=com.android.externalstorage.documents
            //★lastPathSegment=primary:Documents/Remins.txt
            //★path=/document/primary:Documents/Remins.txt
            //★scheme=content/n★schemeSP=//com.android.externalstorage.documents/document/primary:Documents/Remins.txt

            //println("Line:"+ Throwable().stackTrace[0].lineNumber +" ★SetAnnalsDialog★ uri=$uri ★" )
            val butnSave = dialog?.findViewById<View>(R.id.butnSave) as RadioButton
            val butnLoad = dialog?.findViewById<View>(R.id.butnLoad) as RadioButton

            //[回顧録データを退避する]とき
            if ( butnSave.isChecked ) {
                //println("Line:"+ Throwable().stackTrace[0].lineNumber +" ★SetAnnalsDialog★ 回顧録データを退避する ★" )

                //回顧録データを退避する
                saveRemins()
            }

            //[回顧録データを復元する]とき
            if ( butnLoad.isChecked ) {
                //println("Line:"+ Throwable().stackTrace[0].lineNumber +" ★SetAnnalsDialog★ 回顧録データを復元する ★" )

                //回顧録データを復元する
                restoreRemins()
            }

            //ダイアログを閉じる
            dismiss()
            //println("Line:" + Throwable().stackTrace[0].lineNumber +" ★SetAnnalsDialog★ ダイアログを閉じた ★")
        }
//==================================================================================================

        //引数を受け取る
        //val args = arguments?.getString("test").toString()
        //val tag = tag.toString()
        //println("Line:"+ Throwable().stackTrace[0].lineNumber +" ★SetAnnalsDialog★ args=$args tag=$tag★" )

        //dbに接続する
        db = DatabaseHelper(context)

        //ユーザー定数を読み込む
        born = sharedPref.getString("born", "").toString()
        tmod = "" //(初期化の遷移先フラグとして使う)
        //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★SetAnnalsDialog★ born=$born tmod=$tmod")

        //退避ファイルのタイムスタンプを取得する
        val (timeStamp, _) = du.getTimeStamp() //timeStamp, size
        //println("Line:"+ Throwable().stackTrace[0].lineNumber +" ★SetAnnalsDialog★timeStamp=$timeStamp size=$size")

        //ダイアログを開く
        val dialog = Dialog(requireActivity())
        dialog.setContentView(R.layout.dialog_set_annals)

        //プログレスバーを非表示にする
        //val annProgress = dialog.findViewById<View>(R.id.annProgress)
        //annProgress.isVisible = false

        //縦横の向きと機種(タブレット)を特定する
        val tablet = context?.resources?.getBoolean(R.bool.is_tablet) // true or false
        val aspect = resources.configuration.orientation // 1:縦 2:横
        val metrics = resources.displayMetrics
      //val dpi = resources.displayMetrics.densityDpi
        val lp = dialog.window!!.attributes
        //println("Line:"+ Throwable().stackTrace[0].lineNumber +" ★SetAnnalsDialog★ tablet=$tablet aspect=$aspect metrics=$metrics dpi=$dpi metrics.widthPixels=${metrics.widthPixels} lp=$lp lp.width=${lp.width}")

        //ダイアログの表示幅を設定する
        if (tablet == true) {
            lp.width = 960
        } else {

        /* API一覧表
            Pixel 3        Android 8.0  Oreo     API 26 pwidth=720  height=1184 dpi=320 densty=2.0
            SHARP SH-M05   Android 8.0  Oreo     API 26 pwidth=1080 height=1776 dpi=480 densty=3.0
                           Android 8.1  OreoMR   API 27
                           Android 9.0  Pie      API 28
            Pixel 4        Android 10.0 Quince   API 29 pwidth=1080 height=1794 dpi=420 densty=2.625
            Huawei BLA-L29 Android 10.0 Quince   API 29 pwidth=1080 height=2160 dpi=480
                           Android 11.0 RedVel.. API 30
                           Android 12.0 SnowCo.. API 31
            Pixel 6        Android 12.L SnowCo.. API 32 pwidth=1080 height=1878 dpi=420 densty=2.625
            ASUS_I006D     Android 13.0 Tiramisu API 33 pwidth=1080 height=2220 dpi=440 densty=2.75
            Pixel 6        Android 13.0 Upside.. API 33 pwidth=1080 height=2337 dpi=420 densty=2.625
            Pixel 7        Android 14.0 Upside.. API 34
            OPPO_CPH2603   Android 14.0 Upside.. API 34
        */

            //縦向きのとき
            if (aspect == Configuration.ORIENTATION_PORTRAIT) { // 1:縦
                //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★SetAnnalsDialog★ aspect=$aspect:縦")

                lp.width = (metrics.widthPixels * 0.9).toInt()
                //println("Line:"+ Throwable().stackTrace[0].lineNumber +" ★SetAnnalsDialog★ lp.width=${lp.width}")

                //スクロール余白を消去する
                dialog.findViewById<View>(R.id.setAnnalsSpace).visibility = View.GONE

            } else { // 2:横
                //println("Line:"+ Throwable().stackTrace[0].lineNumber +" ★SetAnnalsDialog★ aspect=$aspect:横 metrics.density=${metrics.density} metrics.widthPixels=${metrics.widthPixels}")

                lp.width = metrics.widthPixels
                if ( metrics.density<=2.0 ) {
                    //println("Line:"+ Throwable().stackTrace[0].lineNumber +" ★SetAnnalsDialog★ aspect=$aspect:横 metrics.density=${metrics.density} metrics.widthPixels=${metrics.widthPixels}")
                    lp.width = (metrics.widthPixels * 0.403).toInt() //Pixel3 Android8

                }else if ( metrics.density>2.0 && metrics.density<3.0 ) {
                    //println("Line:"+ Throwable().stackTrace[0].lineNumber +" ★SetAnnalsDialog★ aspect=$aspect:横 metrics.density=${metrics.density} metrics.widthPixels=${metrics.widthPixels}")
                    lp.width = (metrics.widthPixels * 0.403).toInt() //Pixel4 Android10

                }else{
                    //println("Line:"+ Throwable().stackTrace[0].lineNumber +" ★SetAnnalsDialog★ aspect=$aspect:横 metrics.density=${metrics.density} metrics.widthPixels=${metrics.widthPixels}")
                    lp.width = (metrics.widthPixels * 0.55).toInt()  //other
                }
                //println("Line:"+ Throwable().stackTrace[0].lineNumber +" ★SetAnnalsDialog★ lp.width=${lp.width}")

                //スクロール余白を表示する
                dialog.findViewById<View>(R.id.setAnnalsSpace).visibility = View.VISIBLE

            } //if (aspect

        } //if (tablet

        dialog.window!!.attributes = lp
        //println("Line:"+ Throwable().stackTrace[0].lineNumber +" ★SetAnnalsDialog★ダイアログの大きさを設定した。")

        //じぶん年表の初期化の要素を取得する
        val butnInit = dialog.findViewById<View>(R.id.butnInit) as RadioButton
        val instBirth = dialog.findViewById<View>(R.id.instBirth)
        val butnInptBirth = dialog.findViewById<View>(R.id.butnInptBirth) as RadioButton
        val butnDeltBirth = dialog.findViewById<View>(R.id.butnDeltBirth) as RadioButton
        val editBirth = dialog.findViewById<View>(R.id.editBirth)

        //初期の非表示を設定する
        butnInptBirth.visibility = View.GONE
        butnDeltBirth.visibility = View.GONE
        instBirth.visibility = View.GONE
        editBirth.visibility = View.GONE

        //じぶん年表データが読み込まれていないとき
        if ( annalsData=="" ) {
            butnDeltBirth.isEnabled = false //[誕生日を消して年表を削除する]ボタン
        }

        //回顧録データ維持の要素を取得する
        val butnMint = dialog.findViewById<View>(R.id.butnMint) as RadioButton
        val mintGroup = dialog.findViewById<View>(R.id.mintGroup)
      //val textMint = dialog.findViewById<View>(R.id.textMint) as TextView
        val textSave = dialog.findViewById<View>(R.id.textSave) as TextView
        val textAftr = dialog.findViewById<View>(R.id.textAftr) as TextView
        val butnSave = dialog.findViewById<View>(R.id.butnSave) as RadioButton
        val butnLoad = dialog.findViewById<View>(R.id.butnLoad) as RadioButton
        val textRems = dialog.findViewById<View>(R.id.textRems) //as TextView
        textSave.text = context?.getString(R.string.phoneChange) //メッセージ(機種変更の...)

        //じぶん年表データが読み込まれていなくて退避データもないとき
        if ( annalsData=="" && timeStamp=="" ) {
            butnMint.isEnabled = false //[回顧録データを維持する]ボタン
        } else {
            butnMint.isEnabled = true
        }

        //タイムスタンプが読み込まれていないとき (回顧録データが退避されていないとき)
        if( timeStamp=="" ) {
            textAftr.visibility = View.VISIBLE //メッセージ(退避した後... or タイムスタンプ)
            textSave.visibility = View.GONE    //メッセージ(機種変更の...)
        } else {
            textAftr.visibility = View.GONE
            textSave.visibility = View.GONE
        }

        //apiに応じてSAF説明を切り替える
        if ( Build.VERSION.SDK_INT in 26..32 ) { //---------------------------------- api32以下
            textRems.visibility = View.GONE
        } else if ( Build.VERSION.SDK_INT >= Build.VERSION_CODES.TIRAMISU ) { //----------- api33以上
            textRems.visibility = View.VISIBLE
        }

        //回顧録データ維持の初期の非表示を設定する
        mintGroup.isEnabled = false
        butnSave.visibility = View.GONE
        butnLoad.visibility = View.GONE
        textRems.visibility = View.GONE //ファイル選択の指示(SAF)

        //[実行]ボタンを取得する
        val buttonExec = dialog.findViewById<Button>(R.id.buttonExec)
        val buttonCanc = dialog.findViewById<Button>(R.id.buttonCanc)
        val buttonOkok = dialog.findViewById<Button>(R.id.buttonOkok)
        buttonExec.isEnabled = false
        buttonOkok.visibility = View.GONE //OK:非表示

        /*//ボタンの色を設定する
        val drawable = GradientDrawable()
        drawable.color = ColorStateList.valueOf(Color.parseColor("#99eeff")) //ボタンの色
        drawable.setStroke(6, Color.parseColor("#77ccff")) //輪郭
        drawable.cornerRadius = 10f //角丸
        buttonExec.background = drawable
        */

        //[じぶん年表を作成(初期化)する]がクリックされたとき ★★★
        butnInit.setOnClickListener {
            //println("Line:" + Throwable().stackTrace[0].lineNumber +" ★SetAnnalsDialog★[じぶん年表を作成(初期化)する]")

            //[回顧録データを維持する]ボタンのチェックを外す
            butnMint.isChecked = false

            //ファイル選択の説明を隠す
            textRems.visibility = View.GONE //ファイル選択の指示(SAF)
            buttonExec.visibility = View.VISIBLE //実行:表示
            buttonOkok.visibility = View.GONE    //OK:非表示

            //じぶん年表の作成と削除のボタンを表示する
            butnInptBirth.visibility = View.VISIBLE
            butnDeltBirth.visibility = View.VISIBLE
            butnInptBirth.isChecked = false
            butnDeltBirth.isChecked = false

            //回顧録データの[退避]と[維持]のボタンを非表示にする
            butnSave.visibility = View.GONE
            butnLoad.visibility = View.GONE
            textAftr.visibility = View.GONE //メッセージ(退避した後... or タイムスタンプ)
            textRems.visibility = View.GONE //ファイル選択の指示(SAF)
            butnLoad.isChecked = false //復元:off

            //[実行]ボタンを無効にする(誕生日を編集した後でonにする)
            buttonExec.isEnabled = false
        } //butnInit

        //[誕生日を入れて作表する]がクリックされたとき
        butnInptBirth.setOnClickListener {

            //選択肢を切り換える
            butnInptBirth.visibility = View.VISIBLE
            butnDeltBirth.visibility = View.VISIBLE
            butnDeltBirth.isChecked = false

            //誕生日の入力欄を表示する
            instBirth.visibility = View.VISIBLE //メッセージ(あなたの誕生日を入力してください。)
            editBirth.visibility = View.VISIBLE //生年月日の入力欄

            //[実行]ボタンを無効にする(誕生日を編集した後でonにする)
            buttonExec.isEnabled = false

            //【関数】誕生日を編集する
            editBirthday(born)
        }

        //[誕生日を消して年表を削除する]がクリックされたとき
        butnDeltBirth.setOnClickListener {

            //選択肢を切り換える
            butnInptBirth.isChecked = false

            //誕生日の入力欄を非表示にする
            instBirth.visibility = View.GONE
            editBirth.visibility = View.GONE

            //[実行]ボタンを有効にする
            buttonExec.isEnabled = true
        }

        //[回顧録データを維持する]がクリックされたとき ★★★
        butnMint.setOnClickListener {
            //println("Line:" + Throwable().stackTrace[0].lineNumber +" ★SetAnnalsDialog★ [回顧録データを維持する]")

            //[じぶん年表を作成(初期化)する]ボタンのチェックを外す
            butnInit.isChecked = false

            //じぶん年表の作成と削除のボタンを非表示にする
            butnInptBirth.visibility = View.GONE
            butnDeltBirth.visibility = View.GONE

            //誕生日の入力欄を非表示にする
            instBirth.visibility = View.GONE
            editBirth.visibility = View.GONE

            //回顧録データの[退避]と[復元]のボタンを表示する
            butnSave.visibility = View.VISIBLE
            butnLoad.visibility = View.VISIBLE
            textAftr.visibility = View.VISIBLE //メッセージ(退避した後... or タイムスタンプ)
            butnSave.isEnabled = true

            //じぶん年表データが読み込まれていないとき
            if ( annalsData == "" ) {
                //println("Line:"+ Throwable().stackTrace[0].lineNumber +" ★SetAnnalsDialog★ じぶん年表データが読み込まれていない。★")

                //[回顧録データを退避する]ボタンを無効にする
                butnSave.isEnabled = false
            }
            //じぶん年表データが読み込まれているとき
            else{

                //[回顧録データを退避する]ボタンを有効にする
                butnSave.isEnabled = true
            }

            //タイムスタンプが取得できているとき
            if( timeStamp != "" ) {

                //[復元]ボタンを有効にする
                butnLoad.isEnabled = true

                //メッセージをタイムスタンプに書き換える
                textAftr.text = timeStamp //タイムスタンプ(yyyy/mm/dd hh:mm:ss)
                textAftr.setTextColor(Color.BLUE)
            }
            //タイムスタンプが取得できてないとき
            else{

                //[復元]ボタンを無効にする
                butnLoad.isEnabled = false

                //メッセージを消去する
                textAftr.text = getString(R.string.afterBackup) //メッセージ(退避した後...)
                textAftr.setTextColor(Color.GRAY)
            }
        }

        //[回顧録データを退避する]がクリックされたとき
        butnSave.setOnClickListener {
            //println("Line:" + Throwable().stackTrace[0].lineNumber +" ★SetAnnalsDialog★ [回顧録データを退避する]がクリックされた。")

            //[退避]ボタンをonにする
            butnLoad.isChecked = false
            butnSave.isChecked = true

            if ( Build.VERSION.SDK_INT in 26..32 ) { //------------------------------ api32以下

                //[実行]ボタンを有効にする
                buttonExec.isEnabled = true
            }
            else if ( Build.VERSION.SDK_INT >= Build.VERSION_CODES.TIRAMISU ) { //Android 13.0 --- api33以上

                //ファイル選択の説明を表示する(ダミーファイルを選択させる)
                textRems.visibility = View.VISIBLE   //ファイル選択の説明
                buttonExec.visibility = View.GONE    //実行:非表示
                buttonOkok.visibility = View.VISIBLE //OK:表示
            }
        }

        //[回顧録データを復元する]がクリックされたとき
        butnLoad.setOnClickListener {
            //println("Line:" + Throwable().stackTrace[0].lineNumber +" ★SetAnnalsDialog★ [回顧録データを復元する]がクリックされた。")

            //[復元]ボタンをonにする
            butnSave.isChecked = false
            butnLoad.isChecked = true

            if ( Build.VERSION.SDK_INT in 26..32 ) { //------------------------------ api32以下

                //[実行]ボタンを有効にする
                buttonExec.isEnabled = true
            }
            else if ( Build.VERSION.SDK_INT >= Build.VERSION_CODES.TIRAMISU ) { //Android 13.0 --- api33以上

                //ファイル選択の説明を表示する
                textRems.visibility = View.VISIBLE   //ファイル選択の説明
                buttonExec.visibility = View.GONE    //実行:非表示
                buttonOkok.visibility = View.VISIBLE //OK:表示
            }
        }

        //[実行]ボタンがクリックされたとき
        buttonExec.setOnClickListener {
            //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★SetAnnalsDialog★ [実行]ボタンがクリックされた。★")

            //[じぶん年表を初期化する]とき
            if (butnInit.isChecked) {

                //[誕生日を入れて作表する]とき
                if (butnInptBirth.isChecked) {

                    //ユーザーの誕生日を取得する
                    val yy = dialog.findViewById<TextView>(R.id.inpYY).text.toString()
                    var mm = dialog.findViewById<TextView>(R.id.inpMM).text.toString()
                    if (mm.length == 1) {
                        mm = "0$mm"
                    }
                    var dd = dialog.findViewById<TextView>(R.id.inpDD).text.toString()
                    if (dd.length == 1) {
                        dd = "0$dd"
                    }
                    born = "$yy/$mm/$dd"; if (born == "//") {
                        born = ""
                    }
                    //println("Line:" + Throwable().stackTrace[0].lineNumber +" ★SetAnnalsDialog★ born=$born")

                    //再読み込みさせるためにじぶん年表データを消去する
                    annalsData = ""
                }

                //[誕生日を消して年表を削除する]とき
                if (butnDeltBirth.isChecked) {
                    born = ""
                    annalsData = ""
                }

                //初期化の判定フラグを立てる(誕生日が空のときは削除)
                tmod = "initAnnals"

                //ユーザー定数の誕生日を書き替える
                sharedPref.edit().putString("born", born).apply()

                //設定プロセスを振り分ける → initializeAnnalsTable()
                db.initDistributer(tmod)

                //設定の変更を通知する
                done = context?.getString(R.string.doneSetup).toString()
                //println("Line:" + Throwable().stackTrace[0].lineNumber +" ★SetAnnalsDialog★ tmod=$tmod done=$done")

                //ダイアログを閉じる
                dismiss()
                //println("Line:" + Throwable().stackTrace[0].lineNumber +" ★SetAnnalsDialog★ ダイアログを閉じた ★")
            } //[じぶん年表を初期化する]

            //[回顧録データを維持する]とき
            if ( butnMint.isChecked ) {

                //[回顧録データを退避する]とき
                if ( butnSave.isChecked ) { // api32以下のみ (api33以上は[OK]ボタンから)

                    //回顧録データを退避する
                    saveRemins()
                }

                //[回顧録データを復元する]とき
                if ( butnLoad.isChecked ) { // api32以下のみ (api33以上は[OK]ボタンから)
                    //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★SetAnnalsDialog★ 回顧録データを復元する ★")

                    restoreRemins()
                }
            }//[回顧録データを維持する]

            //ダイアログを閉じる
            dismiss()
            //println("Line:" + Throwable().stackTrace[0].lineNumber +" ★SetAnnalsDialog★ ダイアログを閉じた ★")
        }

        //[OK]ボタンがクリックされたとき
        buttonOkok.setOnClickListener { // api33以上のみ
            //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★SetAnnalsDialog★ [OK]ボタンがクリックされた。★")

            //退避ファイルが存在しないとき
           if ( !File(reminsFile).exists() ) {
               //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★SetAnnalsDialog★ reminsFileファイルが存在しない。★")

               //回顧録データのダミーファイルを作成する(削除させるため)
               val text = "Dummy"
               try {
                   val wrtRemStream = FileOutputStream(reminsFile)
                   wrtRemStream.write(text.toByteArray()) //Dummyデータ
                   wrtRemStream.close()
                   //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★MainActivity★Dummyファイルを書き込んだ。★")
               } catch (e: IOException) {
                   e.printStackTrace()
               }
           }

           //退避ファイルを選択して読取りの許可を得る
           getContentLauncher.launch("text/plain")
        }

        //[取消]ボタンがクリックされたとき
        buttonCanc.setOnClickListener {

            //ダイアログを閉じる
            dismiss()
        }

        return dialog
    } //override fun onCreateDialog

    override fun onDestroy() {
        super.onDestroy()
        //println("Line:"+ Throwable().stackTrace[0].lineNumber +" ★SetAnnalsDialog★onDestroy★")

        //設定の変更を通知する
        if ( done!="" ) {
            //println("Line:"+ Throwable().stackTrace[0].lineNumber +" ★SetAnnalsDialog★ done=$done")
            Toast.makeText(activity, done, Toast.LENGTH_LONG).show()
        }

        //親フラグメントに遷移する(じぶん年表を再起動する)
        parentFragmentManager.beginTransaction()
            .replace(R.id.container, FragmentDspAnnals())
            .commit()
    }

//==================================================================================================
    //【関数】回顧録データを退避する
    @RequiresApi(Build.VERSION_CODES.TIRAMISU)
    fun saveRemins() {
        //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★SetAnnalsDialog★ 退避ファイルを削除する。★")

        try {
            File(reminsFile).delete()
            //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★SetAnnalsDialog★ reminsFileファイルを削除した。★")
        } catch (e: IOException) {
            e.printStackTrace()
            //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★SetAnnalsDialog★ reminsFileファイルの削除エラー!!★")
        }

        //[年表データ]と[年齢データ]を保存する
        val rst = db.saveReminsData() //DatabaseHelper Line 600
        //println("Line:" + Throwable().stackTrace[0].lineNumber +" ★SetAnnalsDialog★ rst=$rst")

        //結果を通知する
        done = if( rst ) {
            context?.getString(R.string.remSave).toString()
        }else{
            context?.getString(R.string.doneFalse).toString()
        }

    } //saveRemins

//==================================================================================================
    //【関数】回顧録データを復元する
    @RequiresApi(Build.VERSION_CODES.R)
    fun restoreRemins() {
        var lines: List<String> = listOf()

        //退避ファイルを読む
        try {
            lines = File(reminsFile).bufferedReader().use { it.readLines() }
            //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★SetAnnalsDialog★lines=$lines")

        } catch (e: IOException) {
            e.printStackTrace()
        }

        //年表データをクリアして再読み込みを促す (じぶん年表の表示で判定している)
        annalsData = "restore"

        //回顧録データを読み取れたとき
        if( lines.isNotEmpty() ) {

            //[年表データ]と[年齢データ]を復元する
            val rst = db.restoreReminsData(lines)
            //println("Line:" + Throwable().stackTrace[0].lineNumber +" ★SetAnnalsDialog★ rst=$rst")

            //結果のメッセージをセットする
            done = if (rst) {
                context?.getString(R.string.restore).toString()
            } else {
                context?.getString(R.string.doneFalse).toString()
            }
        }

    } //restoreRemins

//==================================================================================================
    //【関数】誕生日を編集する
    @SuppressLint("NewApi", "CutPasteId", "SimpleDateFormat", "SetTextI18n")
    fun editBirthday(born: String) {

        val minYear = context?.getString(R.string.minYear)?.toInt() //規定の開始年(登録できる最小の生年)
        var yy:String; var yl:Int
        var mm:String; var ml:Int
        var dd:String; var dl:Int
        //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★SetAnnalsDialog★editBirthday★ minYear=$minYear curYear=$curYear born=$born★")

        //[実行][取消]ボタンを取得する
        val buttonExec = dialog?.findViewById<View>(R.id.buttonExec)
      //val buttonCanc = dialog?.findViewById<View>(R.id.buttonCanc)

        //引数(誕生日)を分解して表示する
        val pYY = dialog?.findViewById<TextView>(R.id.inpYY)
        val pMM = dialog?.findViewById<TextView>(R.id.inpMM) //pMM?.isEnabled = false
        val pDD = dialog?.findViewById<TextView>(R.id.inpDD) //pDD?.isEnabled = false
        if (born != "") {
            pYY?.text = born.substring(0, 4)
            pMM?.text = born.substring(5, 7)
            pDD?.text = born.substring(8, 10)
        }
        pYY?.clearFocus()
        pYY?.requestFocus() //テキスト選択は設定しても意味がない　キーボードを開くと邪魔になる


        //[年](yy)のフォーカスが変化したとき ---------------------------------------------------- [年](yy)
        pYY?.setOnFocusChangeListener { _, hasFocus -> //v, hasFocus
            yy = pYY.text.toString()
            yl = pYY.text.length

            //フォーカスを持ったとき
            if ( hasFocus ){
                buttonExec?.isEnabled = false
                (pYY as EditText).setSelection(yl)
            }
            //フォーカスが外れるとき
            else {
                if ( yl != 0 ) {
                    val ychk = du.checkYY(1, minYear, "", yy)
                    if (ychk) {
                        pYY.setTextColor(Color.GRAY)
                    } else {
                        pYY.setTextColor(Color.RED)
                    }
                }
            }
        }

        //[年](yyyy)が入力されたとき
        pYY?.addTextChangedListener(object : TextWatcher {
            override fun beforeTextChanged(s: CharSequence?, start: Int, count: Int, after: Int) { }
            override fun onTextChanged(s: CharSequence?, start: Int, before: Int, count: Int) { }
            override fun afterTextChanged(s: Editable?) {

                //[年](yy)が0桁になったとき
                if( s?.length==0 ) {
                    //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★SetAnnalsDialog★editBirthday after★s=$s l=${s.length}★")
                    pMM?.text = ""
                    pDD?.text = ""
                }

                //[年](yy)が4桁になったとき
                if( s?.length==4 ){
                    //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★SetAnnalsDialog★editBirthday after★s=$s l=${s.length}★")

                    yy = s.toString()
                    val ychk = du.checkYY(1, minYear, "", yy)
                    if ( ychk ) {
                        //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★SetAnnalsDialog★editBirthday★ s=$s l=${s.length} ★")

                        //月と日がすでに入力されているとき
                        mm = pMM?.text.toString()
                        dd = pDD?.text.toString()
                        if( mm.length==2 && dd.length==2 ) {

                            //年月日をチェックする
                            if (du.checkYMD("$yy/$mm/$dd", minYear!!)) {
                                buttonExec?.isEnabled = true
                                pYY.setTextColor(Color.GRAY)
                                closeKeyBord()
                            }
                        } else {
                            pMM?.requestFocus()
                        }
                    }else{
                        pYY.setTextColor(Color.RED)
                        buttonExec?.isEnabled = false
                    }
                }
            }
        })

        //[年](yyyy)からキーボードの完了(Done)ボタンのイベントを検知する
        pYY?.setOnEditorActionListener { _, actionId, _ -> //view, actionId, event

            //[年](yyyy)の入力が終了したとき
            if ( actionId == EditorInfo.IME_ACTION_DONE ) {
                //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★SetAnnalsDialog★ IME_ACTION_DONE★ pYY.text=${pYY.text} ★")

                if( pYY.text.toString().length==4 ) {
                    pMM?.requestFocus()
                }
            }
            return@setOnEditorActionListener true
        } //完了(Done)ボタンのとき

        //[月](mm)のフォーカスが変化したとき ---------------------------------------------------- [月](mm)
        pMM?.setOnFocusChangeListener { _, hasFocus -> //v, hasFocus
            mm = pMM.text.toString()
            ml = pMM.text.length
            //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★SetAnnalsDialog★ mm=$mm ml=$ml ★")

            //フォーカスを持ったとき
            if ( hasFocus ){
                buttonExec?.isEnabled = false
                (pMM as EditText).setSelection(ml)
            }
            //フォーカスが外れるとき
            else {

                if (ml >= 1) {
                    //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★SetAnnalsDialog★ mm=$mm ml=$ml ★")

                    val mchk = du.checkMM(mm)
                    if (mchk) {
                        pMM.setTextColor(Color.GRAY)
                    } else {
                        pMM.setTextColor(Color.RED)
                    }
                }
            }
        }

        //[月](mm)が入力されたとき
        pMM?.addTextChangedListener(object : TextWatcher {
            override fun beforeTextChanged(s: CharSequence?, start: Int, count: Int, after: Int) { }
            override fun onTextChanged(s: CharSequence?, start: Int, before: Int, count: Int) { }
            override fun afterTextChanged(s: Editable?) {

                //[月](mm)が2桁になったとき
                if( s?.length==2 ){
                    //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★SetAnnalsDialog★editBirthday after★s=$s l=${s.length}★")
                    pDD?.requestFocus()
                }
            }
        })

        //[月](mm)からキーボードの完了(Done)ボタンのイベントを検知する
        pMM?.setOnEditorActionListener { _, actionId, _ -> //view, actionId, event

            //[月](mm)の入力が終了したとき
            if ( actionId == EditorInfo.IME_ACTION_DONE ) {
                //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★SetAnnalsDialog★ IME_ACTION_DONE★ pMM.text=${pMM.text} ★")

                if( pMM.text.toString().isNotEmpty() ) {

                    mm = pMM.text.toString()
                    ml = pMM.text.length
                    val mchk = du.checkMM(mm)
                    if ( mchk ) {
                        if (ml == 1) {
                            //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★SetAnnalsDialog★ mm=$mm ml=$ml ★")
                            pMM.text = "0$mm"
                        }
                        pMM.setTextColor(Color.GRAY)
                    }else{
                        pMM.setTextColor(Color.RED)
                        buttonExec?.isEnabled = false
                    }
                }
            } //[月](mm)の入力が(0桁または)1桁で終了したとき
            return@setOnEditorActionListener true
        } //完了(Done)ボタンのとき

        //[日](dd)のフォーカスが変化したとき ---------------------------------------------------- [日](dd)
        pDD?.setOnFocusChangeListener { _, hasFocus -> //v, hasFocus
            yy = pYY?.text.toString(); yl = pYY?.text!!.length
            mm = pMM?.text.toString(); ml = pMM?.text!!.length
            dd = pDD.text.toString();  dl = pDD.text.length
            //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★SetAnnalsDialog★ yy=$yy mm=$mm dd=$dd ★")

            //フォーカスを持ったとき
            if ( hasFocus ){
                buttonExec?.isEnabled = false
                (pDD as EditText).setSelection(dl)
            }
            //フォーカスが外れるとき
            else {

                if (yl == 4 && ml >= 1 && dl >= 1) {
                    //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★SetAnnalsDialog★ yy=$yy mm=$mm dd=$dd ★")

                    //年月日をチェックする
                    if (du.checkYMD("$yy/$mm/$dd", minYear!!)) {
                        buttonExec?.isEnabled = true
                        buttonExec?.requestFocus()
                        closeKeyBord()
                        pDD.setTextColor(Color.GRAY)
                    } else {
                        pDD.setTextColor(Color.RED)
                    }
                }
            }
        }

        //[日](dd)が入力されたとき
        pDD?.addTextChangedListener(object : TextWatcher {
            override fun beforeTextChanged(s: CharSequence?, start: Int, count: Int, after: Int) { }
            override fun onTextChanged(s: CharSequence?, start: Int, before: Int, count: Int) { }
            override fun afterTextChanged(s: Editable?) {

                //[日](dd)が2桁になったとき
                if( s?.length==2 ){
                    //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★SetAnnalsDialog★editBirthday after★s=$s l=${s.length}★")

                    mm = pMM?.text.toString()
                    dd = s.toString()

                    val dchk = du.checkDD(mm, dd)
                    if ( dchk ) {
                        pDD.setTextColor(Color.GRAY)
                    }else{
                        pDD.setTextColor(Color.RED)
                        buttonExec?.isEnabled = false
                    }
                }
            }
        })

        //[日](dd)の入力が終わったとき
        pDD?.setOnEditorActionListener { _, actionId, _ -> //view, actionId, event
            //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★SetAnnalsDialog★editBirthday★ actionId=$actionId ★")
            if (actionId == EditorInfo.IME_ACTION_DONE) {
                yy = pYY?.text.toString()
                mm = pMM?.text.toString(); if (mm.length == 1) { mm = "0$mm"; pMM?.text = mm }
                dd = pDD.text.toString();  if (dd.length == 1) { dd = "0$dd"; pDD.text  = dd }
                (pDD as EditText).setSelection(mm.length)
                //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★SetAnnalsDialog★editBirthday★ yy=$yy mm=$mm dd=$dd born=$yy/$mm/$dd minYear=$minYear★")

                //年月日をチェックする
                if (du.checkDD(mm, dd) && du.checkYMD("$yy/$mm/$dd", minYear!!)) {
                    pDD.setTextColor(Color.GRAY)
                    buttonExec?.isEnabled = true
                    buttonExec?.requestFocus()
                    closeKeyBord()
                } else {
                    //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★SetAnnalsDialog★editBirthday★ yy=$yy mm=$mm dd=$dd born=$yy/$mm/$dd minYear=$minYear★")
                    pDD.setTextColor(Color.RED)
                    buttonExec?.isEnabled = false
                }
            }
            return@setOnEditorActionListener true
        } //[日]

    } //fun editBirthday()

    //キーボードを閉じる --------------------------------------------------------------- キーボードを閉じる
    @SuppressLint("InflateParams")
    private fun closeKeyBord() {

        //キーボードを閉じる
        val imm = context?.getSystemService(Context.INPUT_METHOD_SERVICE) as InputMethodManager
        imm.hideSoftInputFromWindow(view?.windowToken, 0)

        val focusView = dialog?.findViewById<View>(R.id.instBirth)
        imm.hideSoftInputFromWindow( focusView?.windowToken, 0 )

        //[実行]ボタンを有効にする
        val buttonExec = dialog?.findViewById<View>(R.id.buttonExec)
        buttonExec?.isEnabled = true
    } //closeKeyBord

} //class SetAnnalsDialog


