package com.athena.reminiscence.ui.dialog

import android.annotation.SuppressLint
import android.app.Dialog
import android.content.res.Configuration
import android.os.Bundle
import android.view.View
import android.widget.RadioButton
import android.widget.Toast
import androidx.fragment.app.DialogFragment
import com.athena.reminiscence.R
import com.athena.reminiscence.db.DatabaseHelper
import com.athena.reminiscence.sharedPref
import com.athena.reminiscence.ui.main.FragmentDspHowold


//クラスを宣言する
class SetHowoldDialog : DialogFragment() {

    //共用変数を定義する
    private lateinit var db: DatabaseHelper
    private var desp = ""
    private var sort = ""
    private var tmod = ""
    private var done = ""

    //ダイアログを生成する
    @SuppressLint("CutPasteId", "SuspiciousIndentation")
    override fun onCreateDialog(savedInstanceState: Bundle?): Dialog {

        //タグと引数を受け取る
        val args = arguments?.getString("args").toString()
        //println("Line:"+ Throwable().stackTrace[0].lineNumber +" ★SetHowoldDialog args=$args tag=$tag")

        //dbに接続する
        db = DatabaseHelper(context)
        //Toast.makeText(activity, "★SetHowoldDialog★", Toast.LENGTH_SHORT).show()

        //プリファレンスの設定値からユーザー定数を取得する
        desp = sharedPref.getString("desp", "").toString()
        sort = sharedPref.getString("sort", "").toString()

        //ダイアログを開く
        val dialog = Dialog(requireActivity())
        dialog.setContentView(R.layout.dialog_set_howold)

        //縦横の向きと機種(タブレット)を特定する
        val tablet = context?.resources?.getBoolean(R.bool.is_tablet)
        val aspect = resources.configuration.orientation // 1:縦 2:横
        val metrics = resources.displayMetrics
        val lp = dialog.window!!.attributes
        //println("Line:"+ Throwable().stackTrace[0].lineNumber +" ★SetHowoldDialog tablet=$tablet aspect=$aspect metrics=$metrics lp=$lp")

        //ダイアログの表示幅を設定する
        if ( tablet==true ) {
            lp.width = 960
        }else {
            if (aspect == Configuration.ORIENTATION_PORTRAIT) { // 1:縦
                lp.width = (metrics.widthPixels * 0.9).toInt()

            } else { // 2:横
                //println("Line:"+ Throwable().stackTrace[0].lineNumber +" ★SetHowoldDialog★ aspect=$aspect:横 metrics.density=${metrics.density} metrics.widthPixels=${metrics.widthPixels}")

                //Galaxy720:  density=2.0 , metrics.widthPixels=1184
                //SH-M05:     density=3.0 , metrics.widthPixels=1776
                //ASUSU_I006D:density=2.75, metrics.widthPixels=2286
                lp.width = metrics.widthPixels
                if ( metrics.density<=2.0 ) {
                    lp.width = (metrics.widthPixels * 0.9).toInt()

                }else if ( metrics.density>2.0 && metrics.density<3.0 ) {
                    lp.width = (metrics.widthPixels * 0.403).toInt() // 0.65

                }else{
                    lp.width = (metrics.widthPixels * 0.55).toInt()
                }
                //println("Line:"+ Throwable().stackTrace[0].lineNumber +" ★SetHowoldDialog★ lp.width=${lp.width}")
            }
        }
        dialog.window!!.attributes = lp

        //[年齢リストを初期化する]要素を取得する
        val setupHowold = dialog.findViewById<View>(R.id.setupHowold) as RadioButton
        val howInitGroup= dialog.findViewById<View>(R.id.howInitGroup)
        val withSampl= dialog.findViewById<View>(R.id.withSample) as RadioButton
        val nothSampl= dialog.findViewById<View>(R.id.nothSample) as RadioButton
        howInitGroup.visibility = View.GONE

        //[年齢リストの表示範囲を変える]要素を取得する
        val dialogPale = dialog.findViewById<View>(R.id.dialogPale) as RadioButton
        val paleGroup = dialog.findViewById<View>(R.id.paleGroup)
        val pPst = dialog.findViewById<View>(R.id.palePast) as RadioButton
        val pCur = dialog.findViewById<View>(R.id.paleCurr) as RadioButton
        val pBot = dialog.findViewById<View>(R.id.paleBoth) as RadioButton
        paleGroup.visibility = View.GONE

        //[年齢リストの並び順を変える]要素を取得する
        val dialogSort = dialog.findViewById<View>(R.id.dialogSort) as RadioButton
        val sortGroup = dialog.findViewById<View>(R.id.sortGroup)
        val sSeq = dialog.findViewById<View>(R.id.sortInput) as RadioButton
        val sSyl = dialog.findViewById<View>(R.id.sortSylla) as RadioButton
        sortGroup.visibility = View.GONE

        //引数による年齢リストの件数がゼロのとき
        if( args=="0" ){
            //表示範囲と並び順の選択肢を無効にする
            dialogPale.isEnabled = false
            dialogSort.isEnabled = false
        }else{
            (dialog.findViewById<View>(R.id.initPale)).visibility = View.GONE
            (dialog.findViewById<View>(R.id.initSort)).visibility = View.GONE
        }

        //[実行]ボタンを無効にする
        val bExe = dialog.findViewById<View>(R.id.buttonExec)
        bExe.isEnabled = false

        //[年齢リストを初期化する]がクリックされたとき
        setupHowold.setOnClickListener {
            //Toast.makeText(activity, "★[年齢リストを初期化する]★", Toast.LENGTH_LONG).show()

            //選択肢を切り換える
            paleGroup.visibility = View.GONE
            sortGroup.visibility = View.GONE
            howInitGroup.visibility = View.VISIBLE
            withSampl.isChecked = false
            nothSampl.isChecked = false

            //他のチェックを外す
            dialogPale.isChecked = false
            dialogSort.isChecked = false

            //[実行]ボタンを無効にする
            bExe.isEnabled = false
        } //dLst

        //[サンプルデータを付ける。]がクリックされたとき
        withSampl.setOnClickListener { bExe.isEnabled = true }

        //[サンプルデータを付けない。]がクリックされたとき
        nothSampl.setOnClickListener { bExe.isEnabled = true }

        //[年齢リストの表示範囲を変える]がクリックされたとき
        dialogPale.setOnClickListener {
            //println("Line:"+ Throwable().stackTrace[0].lineNumber +" ★SetHowoldDialog desp=$desp sort=$sort")
            //Toast.makeText(activity, "★[年齢リストの表示範囲を変える]★", Toast.LENGTH_LONG).show()

            //選択肢を切り換える
            paleGroup.visibility = View.VISIBLE
            sortGroup.visibility = View.GONE
            howInitGroup.visibility = View.GONE

            //他のチェックを外す
            setupHowold.isChecked = false
            dialogSort.isChecked = false

            //表示範囲をチェックする
            when (desp) {
                "0" -> { pPst.isChecked = true }
                "1" -> { pCur.isChecked = true }
                "2" -> { pBot.isChecked = true }
            }

            //[実行]ボタンを無効にする
            bExe.isEnabled = false
        } //dialogPale

        //[0:過去]がクリックされたとき
        pPst.setOnClickListener { bExe.isEnabled = true }

        //[1:現在]がクリックされたとき
        pCur.setOnClickListener { bExe.isEnabled = true }

        //[2:全部]がクリックされたとき
        pBot.setOnClickListener { bExe.isEnabled = true }

        //[年齢リストの並び順を変える]がクリックされたとき
        dialogSort.setOnClickListener {
            //println("Line:"+ Throwable().stackTrace[0].lineNumber +" ★SetHowoldDialog desp=$desp sort=$sort")
            //Toast.makeText(activity, "★[年齢リストの並び順を変える]★", Toast.LENGTH_LONG).show()

            //選択肢を切り換える
            paleGroup.visibility = View.GONE
            sortGroup.visibility = View.VISIBLE
            howInitGroup.visibility = View.GONE

            //他のチェックを外す
            setupHowold.isChecked = false
            dialogPale.isChecked = false

            //並順モードをチェックする
            if ( sort=="seq" ) { sSeq.isChecked = true }
            if ( sort=="syl" ) { sSyl.isChecked = true }

            //[実行]ボタンを無効にする
            bExe.isEnabled = false
        } //dialogSort

        //[入力順]がクリックされたとき
        sSeq.setOnClickListener { bExe.isEnabled = true }

        //[50音順]がクリックされたとき
        sSyl.setOnClickListener { bExe.isEnabled = true }

        //[実行]ボタンがクリックされたとき
        dialog.findViewById<View>(R.id.buttonExec).setOnClickListener {
            //println("Line:"+ Throwable().stackTrace[0].lineNumber +" ★SetHowoldDialog★[実行]ボタン desp=$desp sort=$sort")

            //初期化が選択されたとき
            if ( setupHowold.isChecked ) {
                if ( withSampl.isChecked ) { sort = "seq" } //サンプルデータを付ける
                if ( nothSampl.isChecked ) { sort = "" }    //サンプルデータを付けない

                //ユーザー定数の並順モードを書き替える
                sharedPref.edit().putString("sort", sort).apply()

                tmod = "initHowold"
                //println("Line:"+ Throwable().stackTrace[0].lineNumber +" ★SetupDialog★ desp=$desp sort=$sort")

                //表示範囲の変更が選択されたとき
            } else if ( dialogPale.isChecked ) {
                if ( pPst.isChecked ) { desp = "0" } // 0:過去
                if ( pCur.isChecked ) { desp = "1" } // 1:現在
                if ( pBot.isChecked ) { desp = "2" } // 2:全部

                //ユーザー定数の表示モードを書き替える
                sharedPref.edit().putString("desp", desp).apply()

                tmod = "dialogPale"

                //並び順の変更が選択されたとき
            } else if ( dialogSort.isChecked ) {
                if ( sSeq.isChecked ) { sort = "seq" } // seq:入力順
                if ( sSyl.isChecked ) { sort = "syl" } // syl:50音順

                //ユーザー定数の並順モードを書き替える
                sharedPref.edit().putString("sort", sort).apply()

                tmod = "dialogSort"
            }
            //println("Line:"+ Throwable().stackTrace[0].lineNumber +" ★SetHowoldDialog★ tmod=$tmod desp=$desp sort=$sort")

            //設定プロセスを振り分ける → initializeHowoldTable
            db.initDistributer(tmod)

            //初期化で一時的に代用した並び順コードを設定する
            if( tmod=="initHowold" ){
                sharedPref.edit().putString("desp", "2").apply()   //初期値
                sharedPref.edit().putString("sort", "seq").apply() //一時的に代用した並び順コードに初期値を設定する
            }

            //設定の変更を通知する
            done = context?.getString(R.string.doneSetup).toString()

            //ダイアログを閉じる
            dismiss()
        }

        //[取消]ボタンがクリックされたとき
        dialog.findViewById<View>(R.id.buttonCanc).setOnClickListener {

            //ダイアログを閉じる
            dismiss()
        }

        return dialog
    } //override fun onCreateDialog

    override fun onDestroy() {
        super.onDestroy()
        //println("Line:"+ Throwable().stackTrace[0].lineNumber +" ★SetHowoldDialog★ onDestroy★ tmod=$tmod")
    /*
        //引数を整える
        val mainConstract: Context? = context
        val intent = Intent(mainConstract, MainActivity::class.java)
        intent.putExtra("trns", tmod)
        intent.putExtra("args", tmod)

        //MainActivityを再起動する(初期化のとき)
        mainConstract?.startActivity(intent)
    */
        //設定の変更を通知する
        if ( done!="" ) {
            //println("Line:"+ Throwable().stackTrace[0].lineNumber +" ★SetAnnalsDialog★ tmod=$tmod done=$done")
            Toast.makeText(activity, done, Toast.LENGTH_LONG).show()
        }

        //println("Line:"+ Throwable().stackTrace[0].lineNumber +" ★SetAnnalsDialog★onDestroy★年齢リストFragmentを再スタートする★")
        //親フラグメントに遷移する(年齢リストを再起動する)
        parentFragmentManager.beginTransaction()
            .replace(R.id.container, FragmentDspHowold())
            .commit()
    } //onDestroy

} //class SetHowoldDialog


