package com.athena.reminiscence.ui.main

import android.annotation.SuppressLint
import android.content.Context
import android.graphics.Color
import android.graphics.drawable.GradientDrawable
import android.os.Bundle
import android.view.*
import android.view.inputmethod.EditorInfo
import android.view.inputmethod.InputMethodManager
import android.widget.Button
import android.widget.ImageView
import android.widget.TextView
import androidx.core.view.MenuHost
import androidx.core.view.MenuProvider
import androidx.fragment.app.Fragment
import androidx.lifecycle.Lifecycle
import com.athena.reminiscence.R
import com.athena.reminiscence.annalsData
import com.athena.reminiscence.db.DatabaseHelper
import com.athena.reminiscence.ui.dialog.EdtAnnalsDialog


//クラスを宣言する
class FragmentEdtAnnals : Fragment() {

    //共用変数を定義する
    private lateinit var db: DatabaseHelper
    private var pwidth = 0
    private var itmOlds = "" //行番号(年齢)
    private var itmAnno = "" //西暦
    private var itmJpns = "" //記号和暦
    private var eraCode = "" //記号
    private var eraJpns = "" //和暦
    private var itmStem = "" //十干
    private var itmBase = "" //よみ
    private var itmZodc = "" //十二支
    private var itmHist = "" //履歴
    private var itmEvnt = "" //時事
    private var itmCodl = "" //特記

    //ビューを生成するとき
    override fun onCreateView( inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle? ): View? {

        //表示エリアをクリアする([じぶん年表]を消去する)(下半分が残ることがある)
        val tbl = view?.findViewById<View>(R.id.annalsTable)
        tbl?.visibility = View.GONE

        //フラグメント幅を物理サイズに合わせる
        pwidth = resources.displayMetrics.widthPixels
        view?.findViewById<View>(R.id.dspHowold)?.layoutParams?.width = pwidth

        //全体のレイアウトを設定する
        return inflater.inflate(R.layout.fragment_edt_annals,container,false)
    } //onViewCreateView

    //ビューの動きを設定するとき
    @SuppressLint("ResourceType", "SetTextI18n", "CutPasteId")
    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★FragmentEdtAnnals★")
/*
        //実機の傾きを取得する
        val orientation = resources.configuration.orientation
        println("Line:"+ Throwable().stackTrace[0].lineNumber +" ★FragmentEdtAnnals★ ★orientation=$orientation★")
*/
/*
        //スタックに積まれた戻り先を取得する
        val frags: MutableList<Fragment> = parentFragmentManager.fragments
        val bc = frags.count()
        //val nm = frags.toString().substring(1, 18)
        //println("Line:"+ Throwable().stackTrace[0].lineNumber +" ★FragmentEdtAnnals★bc=$bc★ frags=$frags★" )

        //スタックが累積したとき
        if ( bc==2 ) {
            val pos = frags.toString().indexOf("EdtAnnalsDialog")

            //ダイアログが開いているとき
            if ( pos!=-1 ) {

                //スタックの後方(戻り先の[ダイアログ])を削除する = [じぶん年表]に戻る
                parentFragmentManager.beginTransaction().remove(frags[1]).commit()

            //ダイアログが開いてないとき
            } else {

                //スタックの前方(戻り先の[年表編集])を削除する = [じぶん年表]に戻る
                parentFragmentManager.beginTransaction().remove(frags[0]).commit()
                //println("Line:"+ Throwable().stackTrace[0].lineNumber +" ★FragmentEdtAnnals★bc=$bc★nm=$nm★ frags=$frags★" )
            }
        }
        // bc==1 回転したとき = 常にFragmentEdtAnnals
*/
        //MenuProviderを設定する
        val menuHost: MenuHost = requireActivity()
        menuHost.addMenuProvider(object : MenuProvider {

            //編集メニューを設定する
            override fun onCreateMenu(menu: Menu, menuInflater: MenuInflater) {

                //回転による重複を避けるためにメユーをクリアする
                //menu.clear()

                //メユーを生成する
                menuInflater.inflate(R.menu.edtbothtmenu, menu)
            }

            //メニューアイコンがクリックされたとき
            override fun onMenuItemSelected(menuItem: MenuItem): Boolean {
                when (menuItem.itemId) {

                    //[説明]アイコンのとき
                    R.id.menu_edit -> {
                        //println("Line:"+ Throwable().stackTrace[0].lineNumber +" ★FragmentEdtAnnals★ menu_edit")

                        //説明ダイアログを開く
                        EdtAnnalsDialog().show(parentFragmentManager, "iconEditAnnals")
                    }
                }
                return true
            }
        }, viewLifecycleOwner, Lifecycle.State.RESUMED)

        //引数を受け取る
        val args = arguments?.getString("trns")
        //println("Line:"+ Throwable().stackTrace[0].lineNumber +" ★FragmentEdtAnnals★ trns=$trns★" )

        //引数を分解して変数に入れる
        val arg: List<String> = args!!.split("\t")
        //println("Line:"+ Throwable().stackTrace[0].lineNumber +" ★FragmentEdtAnnals★arg[0]=${arg[0]} arg[1]=${arg[1]} arg[2]=${arg[2]} arg[3]=${arg[3]} arg[4]=${arg[4]} arg[5]=${arg[5]} arg[6]=${arg[6]} arg[7]=${arg[7]} arg[8]=${arg[8]} arg[9]=${arg[9]}★")
        //xxxxx = arg[0] //recID
        itmOlds = arg[1] //行番号(年齢)
        itmAnno = arg[2] //西暦
        itmJpns = arg[3] //記号和暦(S24)
        eraCode = arg[3].substring(0,1) //略号(M/T/S/H/R)
        eraJpns = arg[3].substring(1,3) //和暦
        itmStem = arg[4] //十干
        itmBase = arg[5] //よみ
        itmZodc = arg[6] //十二支
        itmHist = arg[7] //履歴
        itmEvnt = arg[8] //時事
        itmCodl = arg[9] //特記
        //println("Line:"+ Throwable().stackTrace[0].lineNumber +" ★FragmentEdtAnnals★\n itmOlds=$itmOlds \n itmAnno=$itmAnno \n eraCode=$eraCode \n itmJpns=$itmJpns \n itmHist=$itmHist \n itmEvnt=$itmEvnt \n itmCodl=$itmCodl")

        //履歴が白寿のとき
        if( itmOlds=="99" ){
            //println("Line:"+ Throwable().stackTrace[0].lineNumber +" ★FragmentEdtAnnals★\n arg[7]=${arg[7]} itmHist=$itmHist")
            val arg7 = arg[7].split("|")
            view.findViewById<TextView>(R.id.editHist).tag = arg[7]
            itmHist = arg7[1]
            //println("Line:"+ Throwable().stackTrace[0].lineNumber +" ★FragmentEdtAnnals★\n tag=${view.findViewById<TextView>(R.id.editHist).tag} itmHist=$itmHist")
        }

        //記号から和暦を取得する
        val eraMap : Map<String, Int> = mapOf("" to 0,  "M" to 1, "T" to 2, "S" to 3, "H" to 4, "R" to 5)
        val eraNmbr = eraMap[eraCode]!! + 1
        val eraArray = resources.getStringArray(R.array.spinnerItems) //元号array
        val eraName = eraArray[eraNmbr]
        //println("Line:" + Throwable().stackTrace[0].lineNumber + " FragmentEdtAnnals★ eraCode=$eraCode eraName=$eraName★")

        //ビューに引数の値を表示する
        view.findViewById<TextView>(R.id.dspOlds).text = itmOlds
        view.findViewById<TextView>(R.id.dspAnno).text = itmAnno
        view.findViewById<TextView>(R.id.lblJpns).text = eraName
        view.findViewById<TextView>(R.id.dspJpns).text = itmJpns
        view.findViewById<TextView>(R.id.lblStem).text = itmStem
        view.findViewById<TextView>(R.id.lblBase).text = "($itmBase)"
        view.findViewById<TextView>(R.id.lblZdac).text = itmZodc

        val hist = view.findViewById<TextView>(R.id.editHist); hist.text = itmHist
        val evnt = view.findViewById<TextView>(R.id.editEvnt); evnt.text = itmEvnt
        val codl = view.findViewById<TextView>(R.id.editCodl); codl.text = itmCodl
        val gHist = hist.background as GradientDrawable; gHist.setColor(Color.WHITE); gHist.setStroke(4, Color.LTGRAY) //背景(白) 枠線(灰)
        val gEvnt = evnt.background as GradientDrawable; gEvnt.setColor(Color.WHITE); gEvnt.setStroke(4, Color.LTGRAY) //背景(白) 枠線(灰)
        val gCodl = codl.background as GradientDrawable; gCodl.setColor(Color.WHITE); gCodl.setStroke(4, Color.LTGRAY) //背景(白) 枠線(灰)

        //白寿の変更を無効にする
        if( itmOlds=="99" ){
            hist.isEnabled = false
            gHist.setColor(Color.LTGRAY) //背景(灰)
        }

        //保存ボタンを無効にする
        val btnExec = view.findViewById<Button>(R.id.btnAnnalsExec)
        btnExec.isEnabled = false

        //じぶん年表を編集する
        editAnnalsData()

        //プログレスバーを非表示にする
        view.findViewById<View>(R.id.annalsEdtProgressBar).visibility = View.INVISIBLE
    } //onViewCreated
/*
    //フラグメントを破棄する
    @RequiresApi(Build.VERSION_CODES.O)
    override fun onDestroy(){
        super.onDestroy()
        println("Line:"+ Throwable().stackTrace[0].lineNumber +" ★FragmentEdtAnnals ★★★ onDestroy ★★★" )
    }
*/
//==================================================================================================

    //じぶん年表を編集する
    @SuppressLint("CutPasteId", "SuspiciousIndentation")
    private fun editAnnalsData() {

        //dbに接続する
        db = DatabaseHelper(context)

        //設定されているViewを取得する
        val hist = view?.findViewById<TextView>(R.id.editHist)
        val evnt = view?.findViewById<TextView>(R.id.editEvnt)
        val codl = view?.findViewById<TextView>(R.id.editCodl)
        val clearHist = view?.findViewById<ImageView>(R.id.clearHist)
        val clearEvnt = view?.findViewById<ImageView>(R.id.clearEvnt)
        val clearCodl = view?.findViewById<ImageView>(R.id.clearCodl)
        val btnExec = view?.findViewById<Button>(R.id.btnAnnalsExec)
        val btnDelt = view?.findViewById<Button>(R.id.btnAnnalsDelt)

        //履歴の消去ボタンを無効にする
        if( itmOlds=="99" ){
            clearHist?.isEnabled = false
        }

        //マルチラインでのactionDoneを有効にする
        codl?.inputType = EditorInfo.TYPE_TEXT_FLAG_MULTI_LINE
        codl?.setHorizontallyScrolling(false)
        codl?.maxLines = Integer.MAX_VALUE

        //履歴と時事と特記のすべてが空のとき
        if ( hist?.text.toString()=="" && evnt?.text.toString()=="" && codl?.text.toString()=="" ) {

            //消去ボタンを無効にする
            btnDelt?.isEnabled = false
            btnDelt?.setBackgroundColor(Color.rgb(226, 226, 226))
            btnDelt?.setTextColor(Color.GRAY)
        }

        //[履歴]リスナーを設定する
        hist?.setOnFocusChangeListener { _, _ ->
            btnExec?.isEnabled = false
        }
        hist?.setOnEditorActionListener { _, actionId, _ ->

            //[履歴]の入力が終わったとき
            if (actionId == EditorInfo.IME_ACTION_DONE) {

                //入力された履歴を取得する
                val inpHist = hist.text.toString()
                //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★FragmentEdtAnnals★ itmHist=$itmHist inpHist=$inpHist")

                //キーボードを閉じる
                val imm = context?.getSystemService(Context.INPUT_METHOD_SERVICE) as InputMethodManager
                imm.hideSoftInputFromWindow(view?.windowToken, 0)

                //履歴が変更されたとき(保存ボタンを有効にする)
                if ( itmHist!=inpHist ) {
                    btnExec?.isEnabled = true
                    btnDelt?.isEnabled = true
                    btnDelt?.setBackgroundColor(Color.rgb(100, 149, 237))
                    btnDelt?.setTextColor(Color.WHITE)
                }
            }
            true
        }

        //[時事]リスナーを設定する
        evnt?.setOnFocusChangeListener { _, _ ->
            btnExec?.isEnabled = false
        }
        evnt?.setOnEditorActionListener { _, actionId, _ ->

            //[時事]の入力が終わったとき
            if (actionId == EditorInfo.IME_ACTION_DONE) {

                //入力された時事を取得する
                val inpEvnt = evnt.text.toString()
                    //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★FragmentEdtAnnals★ itmEvnt=$itmEvnt inpEvnt=$inpEvnt")

                //キーボードを閉じる
                val imm = context?.getSystemService(Context.INPUT_METHOD_SERVICE) as InputMethodManager
                imm.hideSoftInputFromWindow(view?.windowToken, 0)

                //時事が変更されたとき(保存ボタンを有効にする)
                if ( itmEvnt!=inpEvnt ) {
                    btnExec?.isEnabled = true
                    btnDelt?.isEnabled = true
                    btnDelt?.setBackgroundColor(Color.rgb(100, 149, 237))
                    btnDelt?.setTextColor(Color.WHITE)
                }
            }
            true
        }

        //[特記]リスナーを設定する
        codl?.setOnFocusChangeListener { _, _ ->
            btnExec?.isEnabled = false
        }
        codl?.setOnEditorActionListener { _, actionId, _ ->

            //[特記]の入力が終わったとき
            if (actionId == EditorInfo.IME_ACTION_DONE) { // actionId = 6

                //入力された特記を取得する
                val inpCodl = codl.text.toString()
                //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★FragmentEdtAnnals★ inpCodl=$inpCodl")

                //キーボードを閉じる
                val imm = context?.getSystemService(Context.INPUT_METHOD_SERVICE) as InputMethodManager
                imm.hideSoftInputFromWindow(view?.windowToken, 0)

                //特記が変更されたとき(保存ボタンを有効にする)
                if ( itmCodl!=inpCodl ) {
                    btnExec?.isEnabled = true
                    btnDelt?.isEnabled = true
                    btnDelt?.setBackgroundColor(Color.rgb(100, 149, 237))
                    btnDelt?.setTextColor(Color.WHITE)
                }
            }
            true
        }

        //[クリア]ボタンがクリックされたとき
        clearHist?.setOnClickListener {

            //表示しているデータを消去する
            view?.findViewById<TextView>(R.id.editHist)?.text = ""
            btnExec?.isEnabled = true
        }
        clearEvnt?.setOnClickListener {
            view?.findViewById<TextView>(R.id.editEvnt)?.text = ""
            btnExec?.isEnabled = true
        }
        clearCodl?.setOnClickListener {
            view?.findViewById<TextView>(R.id.editCodl)?.text = ""
            btnExec?.isEnabled = true
        }

        //[取消]ボタンがクリックされたとき
        view?.findViewById<Button>(R.id.btnAnnalsCanc)?.setOnClickListener {
            //println("Line:"+ Throwable().stackTrace[0].lineNumber +" ★FragmentEdtAnnals★ Button: [取消] ")

            //プログレスバーを表示する
            requireView().findViewById<View>(R.id.annalsEdtProgressBar).visibility = View.VISIBLE

            //フラグメントを再スタートする
            restartAnnals("btnAnnalsCanc")
        } //btnAnnalsCanc

        //[保存]ボタンがクリックされたとき
        btnExec?.setOnClickListener {
            //println("Line:"+ Throwable().stackTrace[0].lineNumber +" Button: [保存]")

            //プログレスバーを表示する
            requireView().findViewById<View>(R.id.annalsEdtProgressBar).visibility = View.VISIBLE

            //編集されたデータを読み取る
            var editHist = view?.findViewById<TextView>(R.id.editHist)?.text.toString()
            val editEvnt = view?.findViewById<TextView>(R.id.editEvnt)?.text.toString()
            val editCodl = view?.findViewById<TextView>(R.id.editCodl)?.text.toString()
            if( itmOlds=="99" ){
                editHist = view?.findViewById<TextView>(R.id.editHist)?.tag.toString()
                //println("Line:"+ Throwable().stackTrace[0].lineNumber +" ★ editHist=$editHist")
            }
            //println("Line:"+ Throwable().stackTrace[0].lineNumber +" ★ itmOlds=$itmOlds itmAnno=$itmAnno itmJpns=$itmJpns itmStem=$itmStem itmZodc=$itmZodc editHist=$editHist editEvnt=$editEvnt editCodl=$editCodl")

            //レコードを更新する
            db.updateAnnalsData(itmOlds.toInt(), itmAnno, itmJpns, itmStem, itmBase, itmZodc, editHist, editEvnt, editCodl)

            //年表データをクリアして再読み込みを促す
            annalsData = ""

            //年表フラグメントに切り替える
            restartAnnals("btnAnnalsExec")

        } //btnAnnalsExec

        //[消去]ボタンがクリックされたとき
        btnDelt?.setOnClickListener {
            //println("Line:"+ Throwable().stackTrace[0].lineNumber +" ★FragmentEdtAnnals★ Button: [消去] ")

            //表示しているデータを消去する
            view?.findViewById<TextView>(R.id.editHist)?.text = ""
            view?.findViewById<TextView>(R.id.editEvnt)?.text = ""
            view?.findViewById<TextView>(R.id.editCodl)?.text = ""

            //機能(消去)ボタンを無効にする
            btnDelt.setBackgroundColor(Color.rgb(226, 226, 226))
          //btnDelt.setTextColor(Color.GRAY)
            btnDelt.isEnabled = false

            //機能(保存)ボタンを有効にする
            btnExec?.isEnabled = true
        } //btnAnnalsDele

    } //editAnnalsData

//==================================================================================================

    //Fragmentを再スタートする
    private fun restartAnnals(btn:String){
        //println("Line:"+ Throwable().stackTrace[0].lineNumber +" ★Edit ★★★ restartAnnals btn=$btn ★★★" )

        //引数をセットする
        val bundle = Bundle()
        bundle.putString("trns", btn)

        //[じぶん年表]に戻る
        val dspAnnalsFragment = FragmentDspAnnals()
        dspAnnalsFragment.arguments = bundle

        parentFragmentManager.beginTransaction()
          //.add(R.id.container, dspAnnalsFragment)
            .replace(R.id.container, dspAnnalsFragment)
            .commit()
    }

} //class Fragment_Edt_Annals