package com.athena.reminiscence.ui.main

import android.annotation.SuppressLint
import android.content.Context.*
import android.content.res.Configuration
import android.graphics.Color
import android.os.Build
import android.os.Bundle
import android.text.Editable
import android.text.TextWatcher
import android.view.*
import android.view.inputmethod.EditorInfo
import android.view.inputmethod.InputMethodManager
import android.widget.*
import androidx.annotation.RequiresApi
import androidx.core.view.MenuHost
import androidx.core.view.MenuProvider
import androidx.core.view.WindowInsetsCompat
import androidx.fragment.app.Fragment
import androidx.lifecycle.Lifecycle
import com.athena.reminiscence.R
import com.athena.reminiscence.db.DatabaseHelper
import com.athena.reminiscence.db.DateUtility
import com.athena.reminiscence.ui.dialog.EdtHowoldDialog


//クラスを宣言する
class FragmentEdtHowold : Fragment() {

    //共用変数を定義する
    private lateinit var db: DatabaseHelper
    private val du = DateUtility()
    private var pwidth = 0
    private var itmSeqs = "" //sequence number(temporarily)
    private var itmRcid = "" //record number
    private var itmName = "" //氏名(名称)
    private var itmKana = "" //かな
    private var itmUpnm = "" //元号(開始日)
    private var itmDnnm = "" //元号(終了日)
    private var itmBegn = "" //開始日
    private var itmTerm = "" //終了日
    private var itmZodc = "" //干支
    private var iEra = arrayOf("","","","","","","") // "  ","西暦","明治","大正",...
    private var uix = 0 //spinner selection
    private var dix = 0 //spinner selection
    private var minYear = 0

    //ビューを生成するとき
    override fun onCreateView( inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle? ): View? {

        //フラグメント幅を物理サイズに合わせる
        pwidth = resources.displayMetrics.widthPixels
        view?.findViewById<View>(R.id.dspHowold)?.layoutParams?.width = pwidth

        //全体のレイアウトを設定する
        return inflater.inflate(R.layout.fragment_edt_howold,container,false)
    }

    //ビューの動きを設定するとき
    @SuppressLint("CutPasteId", "SetTextI18n", "ResourceAsColor")
    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

        /*
        //スタックに積まれた戻り先を取得する(ここでは常にbc=1となる)
        val frags: MutableList<Fragment> = parentFragmentManager.fragments
        val bc = frags.count()
        val nm = frags.toString().substring(1, 18)
        println("Line:"+ Throwable().stackTrace[0].lineNumber +" ★FragmentEdtHowold★bc=$bc★nm=$nm★ frags=$frags★" )
        */

        //MenuProviderを設定する
        val menuHost: MenuHost = requireActivity()
        menuHost.addMenuProvider(object : MenuProvider {

            //編集メニューを設定する
            override fun onCreateMenu(menu: Menu, menuInflater: MenuInflater) {

                //回転による重複を避けるためにメユーをクリアする
                //menu.clear()

                //メユーを生成する
                menuInflater.inflate(R.menu.edtbothtmenu, menu)
            }

            //メニューアイコンがクリックされたとき
            override fun onMenuItemSelected(menuItem: MenuItem): Boolean {
                when (menuItem.itemId) {

                    //[説明]アイコンのとき
                    R.id.menu_edit -> {

                        //説明ダイアログを開く
                        EdtHowoldDialog().show(parentFragmentManager, "iconEditHowold")
                    }
                }
                return true
            }

        }, viewLifecycleOwner, Lifecycle.State.RESUMED)
        //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★FragmentEdtHowold★")

        //氏名ビューを取得する
        val uTL = view.findViewById<TextView>(R.id.edtHowoldTitle)
        val uSQ = view.findViewById<TextView>(R.id.lblSeqs)
        val uNM = view.findViewById<TextView>(R.id.edtName)
        val uKN = view.findViewById<TextView>(R.id.editKana); uKN.isEnabled = false

        //スピンナーを取得する
        val uSS = view.findViewById<Spinner>(R.id.spinner_up)
        val dSS = view.findViewById<Spinner>(R.id.spinner_dn)

        //横向きのとき(縦位置を補正する)(ここで回転を判別しているわけではない)
        val orientation = resources.configuration.orientation
        if (orientation == Configuration.ORIENTATION_LANDSCAPE) {
            //画面中央を0としてX座標とY座標をFloatで指定する
            uSS.translationY = 0F
            dSS.translationY = 0F
            //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★FragmentEdtHowold★ 縦位置の補正")
        }

        //元号を取得する
        iEra = resources.getStringArray(R.array.spinnerItems) // "  ","西暦","明治","大正",...

        //アダプターを設定する
        val adapter = activity?.let { ArrayAdapter(it, R.layout.spinner_item, iEra) } //元号
        uSS.adapter = adapter//; uSS.isFocusable = true; uSS.isFocusableInTouchMode = true
        dSS.adapter = adapter//; dSS.isFocusable = true; dSS.isFocusableInTouchMode = true

        //[日付]スピンナーを無効にする
        uSS.isEnabled = false; uSS.setSelection(0)
        dSS.isEnabled = false; dSS.setSelection(0)

        //開始日を設定する(android:selectAllOnFocus=trueを設定済)
        val uYY = view.findViewById<TextView>(R.id.inpYY_up); uYY.isEnabled = false
        val uMM = view.findViewById<TextView>(R.id.inpMM_up); uMM.isEnabled = false
        val uDD = view.findViewById<TextView>(R.id.inpDD_up); uDD.isEnabled = false
        val uRR = view.findViewById<TextView>(R.id.refRevs_up)
        val uRS = view.findViewById<TextView>(R.id.refStem_up)
        val uRZ = view.findViewById<TextView>(R.id.refZodc_up)

        //終了日を設定する(android:selectAllOnFocus=trueを設定済)
        val dYY = view.findViewById<TextView>(R.id.inpYY_dn); dYY.isEnabled = false
        val dMM = view.findViewById<TextView>(R.id.inpMM_dn); dMM.isEnabled = false
        val dDD = view.findViewById<TextView>(R.id.inpDD_dn); dDD.isEnabled = false
        val dRR = view.findViewById<TextView>(R.id.refRevs_dn)

        //[保存]ボタンを無効にする
        val btnExec = view.findViewById<Button>(R.id.btnExec)
        btnExec.isEnabled = false

        // ↑↑↑↑↑ ここまでビュー設定 ↑↑↑↑↑ ===== ↓↓↓↓↓ ここからデータ設定 ↓↓↓↓↓ ================================

        //dbに接続する
        db = DatabaseHelper(context)

        //引数を受け取る
      //val base = arguments?.getString("base").toString() //未使用
        val args = arguments?.getString("trns")
        //println("Line:"+ Throwable().stackTrace[0].lineNumber +" ★FragmentEdtHowold★ base=$base★" )

        //引数を分解して変数に入れる
        val arg: List<String> = args!!.split(",")
        itmSeqs = arg[0] //行番号
        itmRcid = arg[1] //recID
        itmName = arg[2] //氏名(名称)
        itmKana = arg[3] //かな
        itmUpnm = arg[4] //元号(上)
        itmDnnm = arg[5] //元号(下)
        itmBegn = arg[6] //開始(誕生)日
        itmTerm = arg[7] //終了(満期)日
        itmZodc = arg[8] //; if (itmZodc == "()") { itmZodc = "" }

        minYear = context?.getString(R.string.minYear)?.toInt()!! //登録できるユーザーの最小の生年(明治維新よりも前は元号を特定でないため扱えない)
        //println("Line:"+ Throwable().stackTrace[0].lineNumber +" ★FragmentEdtHowold★\n★itmSeqs=$itmSeqs\n★itmRcid=$itmRcid\n★itmName=$itmName\n★itmKana=$itmKana\n★itmUpnm=$itmUpnm\n★itmDnnm=$itmDnnm\n★itmBegn=$itmBegn\n★itmTerm=$itmTerm\n★itmZodc=$itmZodc\n★minYear=$minYear")

        //アダプターの選択肢を設定する
        uix = iEra.indexOf(itmUpnm); if (uix == -1) { uix = 0 }
        dix = iEra.indexOf(itmDnnm); if (dix == -1) { dix = 0 }
        //println("Line:"+ Throwable().stackTrace[0].lineNumber +" ★FragmentEdtHowold★ uix=$uix itmUpnm=$itmUpnm dix=$dix itmDnnm=$itmDnnm")

        //登録のとき(氏名と開始日が空のとき)
        if ( itmName == "" && itmBegn == "" ) {
            //println("Line:"+ Throwable().stackTrace[0].lineNumber +" ★FragmentEdtHowold★登録 itmName=$itmName itmBegn=$itmBegn itmTerm=$itmTerm")

            //未入力チェックのためにダミー日付を設定する(yy/mm/ddを分割するために空ではまずい)
            itmBegn = "8888/88/88"
            itmTerm = "8888/88/88"

            //[開始日]と[終了日]のスピンナーを無効にする
            uSS.setSelection(0); uSS.isEnabled = false
            dSS.setSelection(0); dSS.isEnabled = false

            //削除ボタンを無効にする
            val btnDele = view.findViewById<Button>(R.id.btnDele)
            btnDele.isEnabled = false

            //[氏名]ビューにフォーカスする
            uSQ.text = "$itmSeqs:"
            uNM.requestFocus()
            //println("Line:"+ Throwable().stackTrace[0].lineNumber +" ★FragmentEdtHowold★登録 itmName=$itmName itmBegn=$itmBegn itmTerm=$itmTerm")
        }

        //修正または削除のとき
        else {
            //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★FragmentEdtHowold★ 修正または削除 itmName=$itmName itmBegn=$itmBegn itmTerm=$itmTerm")

            //氏名を表示する
            uSQ.text = "$itmSeqs:"
            uNM.text = itmName
            uKN.text = itmKana; uKN.isEnabled = true

            //[開始日]と[終了日]のスピンナーを有効にする
            uSS.setSelection(uix); uSS.isEnabled = true
            dSS.setSelection(dix); dSS.isEnabled = true

            //[開始日]と[終了日]の年月日を有効にする
            uYY.isEnabled = true
            uMM.isEnabled = true
            uDD.isEnabled = true
            dYY.isEnabled = true
            dMM.isEnabled = true
            dDD.isEnabled = true

            //[開始日]が空でないとき
            if ( itmBegn != "" ) {
                //println("Line:"+ Throwable().stackTrace[0].lineNumber +" ★FragmentEdtHowold itmBegn=$itmBegn itmTerm=$itmTerm")

                //【関数】日付を表示する(参照日付は西暦と和暦とを逆に表示する)
                dspDateValue("up") // 開始or終了,元号,和暦年,年,月,日
            } //開始日

            //[終了日]が空でないとき
            if ( itmTerm != "" ) {
                //println("Line:"+ Throwable().stackTrace[0].lineNumber +" ★FragmentEdtHowold itmBegn=$itmBegn itmTerm=$itmTerm")

                //【関数】日付を表示する(参照日付は西暦と和暦とを逆に表示する)
                dspDateValue("dn") // 開始or終了,元号,和暦年,年,月,日 dSS.isEnabled
            }

            //タップするまでフォーカスを外す(初期表示での選択を避ける)
            uTL.requestFocus()

        } //修正または削除

        //プログレスバーを非表示にする
        view.findViewById<View>(R.id.howoldEdtProgressBar).visibility = View.INVISIBLE


    // ↑↑↑↑↑ ここまでビュー設定 ↑↑↑↑↑ ===== ↓↓↓↓↓ ここからリスナー設定 ↓↓↓↓↓ ==============================

        //[氏名]リスナーを設定する
        uNM.setOnFocusChangeListener { _, _ ->
            btnExec.isEnabled = false
        }
        uNM.setOnEditorActionListener { _, actionId, _ ->

            //[氏名]の入力が終わったとき
            if ( actionId == EditorInfo.IME_ACTION_DONE ) {

                //入力された氏名を取得する
                val inpName = uNM.text.toString()

                //[氏名]が空でないとき(空は許されない)
                if ( inpName != "" ) {
                    //println("Line:"+ Throwable().stackTrace[0].lineNumber +" ★FragmentEdtHowold inpName=$inpName")

                    //氏名が変更されたとき
                    if ( itmName != inpName ) {
                        //println("Line:"+ Throwable().stackTrace[0].lineNumber +" ★FragmentEdtHowold itmName=$itmName inpName=$inpName")

                        //全角スペースを半角スペースに置換する
                        val name = inpName.replace("　".toRegex(), " ")
                        uNM.text = name
                        itmName = name
                        //println("Line:"+ Throwable().stackTrace[0].lineNumber +" ★FragmentEdtHowold itmName=$itmName")

                        //[開始日]が空でないとき
                        if ( itmBegn != "8888/88/88" && itmBegn != "" ) {
                            //println("Line:"+ Throwable().stackTrace[0].lineNumber +" ★FragmentEdtHowold itmName=$itmName itmBegn=$itmBegn")

                            //キーボードを閉じる
                            closeKeyBord(view)

                            //[保存]ボタンを有効にする
                            btnExec.isEnabled = true
                        }

                        //開始日と終了日が空のとき(登録のとき)
                        if ( itmBegn == "8888/88/88" && itmTerm == "8888/88/88" ) {

                            //[かな]にフォーカスする　(修正のときは[保存]ボタンを有効のままにするためフォーカスしない)
                            uKN.isEnabled = true
                            uKN.requestFocus()

                            //[開始日]スピンナーを有効にする
                            uSS.isEnabled = true
                            uSS.setSelection(uix) //0,1,2,3,4,5,6
                        }
                    }
                }
            }
            true
        }

        //[かな]リスナーを設定する
        uKN.setOnFocusChangeListener { _, _ ->
            btnExec.isEnabled = false
        }
        uKN.setOnEditorActionListener { _, actionId, _ ->

            //[かな]の入力が終わったとき
            if ( actionId == EditorInfo.IME_ACTION_DONE ) {

                //入力されたカナを取得する
                val inpKana = uKN.text.toString()
                //println("Line:"+ Throwable().stackTrace[0].lineNumber +" ★FragmentEdtHowold inpKana=$inpKana")

                //[かな]が変更されたとき([かな]は省略されるかもしれない)
                if ( itmKana != inpKana ) {

                    //全角スペースを半角スペースに置換する
                    val kana = inpKana.replace("　".toRegex(), " ")
                    uKN.text = kana
                    itmKana = kana
                    (uKN as EditText).setSelection(uKN.text.length)
                    uTL.requestFocus() //dummy
                    //println("Line:"+ Throwable().stackTrace[0].lineNumber +" ★FragmentEdtHowold inpKana=$inpKana itmKana=$itmKana")

                    //[開始日]が空でないとき
                    if ( itmBegn != "8888/88/88" && itmBegn != "" ) {

                        //[保存]ボタンを有効にする
                        btnExec.isEnabled = true
                        btnExec.requestFocus()
                    }
                    closeKeyBord(view)
                }

                //[開始日]スピンナーを有効にする
                uYY.isEnabled = true //**************************
                uSS.isEnabled = true
                uSS.requestFocus()
                if ( itmBegn == "8888/88/88" ) {
                    uSS.performClick() //開く
                }
            }
            true
        }

        // ↑↑↑↑↑ ここまで氏名ビュー ↑↑↑↑↑ ===== ↓↓↓↓↓ ここから日付ビュー ↓↓↓↓↓ ================================

        //開始日:[元号]リスナーを実装する
        uSS.onItemSelectedListener = object : AdapterView.OnItemSelectedListener {

            //元号が選択されたとき
            @RequiresApi(Build.VERSION_CODES.R)
            override fun onItemSelected(parent: AdapterView<*>?, view: View?, position: Int, id: Long ) {
                //println("Line:"+ Throwable().stackTrace[0].lineNumber +" ★FragmentEdtHowold [開始日]リスナー \n position=$position \n itmRcid=$itmRcid \n itmName=$itmName \n itmUpnm=$itmUpnm \n itmDnnm=$itmDnnm \n itmBegn=$itmBegn \n itmTerm=$itmTerm")

                //[元号]スピンナーの選択値を読み取る
                uix = uSS.selectedItemPosition

                //元号が未選択または再選択されたとき(西暦･和暦または元号を変更したとき)
                if ( uix == 0 || itmUpnm != iEra[uix] ) { // 登録されている元号 != 開始日の元号(空白)
                    //println("Line:"+Throwable().stackTrace[0].lineNumber+" ★FragmentEdtHowold onItemSelected-in★ itmBegn=$itmBegn uix=$uix itmUpnm=$itmUpnm iEra[uix]=${iEra[uix]}")

                    //開始日の設定値をクリアして入力不可にする(再入力させる)
                    uTL.requestFocus() //dummy
                    btnExec.isEnabled = false
                    itmBegn = "8888/88/88"
                    itmUpnm = ""
                    uYY.text = ""; uYY.isEnabled = true
                    uMM.text = ""; uMM.isEnabled = false
                    uDD.text = ""; uDD.isEnabled = false
                    uRR.text = "" //参照(反転)表示
                    uRS.text = "" //十干よみ
                    uRZ.text = "" //干支
                    //println("Line:"+Throwable().stackTrace[0].lineNumber+" ★FragmentEdtHowold★ uix=$uix itmUpnm=$itmUpnm iEra[uix]=${iEra[uix]}")

                    //終了日の設定値をクリアして入力不可にする(再入力させる)
                    dSS.setSelection(0)
                    dSS.isEnabled = false
                    dix = 0
                    if( itmTerm != "8888/88/88" ){ itmTerm = "" }
                    itmDnnm = ""
                    dYY.text = ""; dYY.isEnabled = false
                    dMM.text = ""; dMM.isEnabled = false
                    dDD.text = ""; dDD.isEnabled = false
                    dRR.text = "" //参照(反転)表示
                    //println("Line:"+Throwable().stackTrace[0].lineNumber+" ★FragmentEdtHowold★ uix=$uix itmDnnm=$itmDnnm iEra[uix]=${iEra[uix]}")
                }

                //有効な値が選択されたとき
                if ( uix != 0 ) {

                    //元号を取得する
                    itmUpnm = iEra[uix] // "  ","西暦","明治","大正","昭和","平成","令和"
                    //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★FragmentEdtHowold itmUpnm=$itmUpnm")

                    //年(yy)が空のとき
                    if ( uYY.text.toString() == "" ) {
                        //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★FragmentEdtHowold uix=$uix uYY.text=${uYY.text}")

                        //[年]の入力を有効にする
                      //uYY.isEnabled = true 元号の選択で設定済
                        uYY.requestFocus()

                        //キーボードを開く (Android 10.0 : Quince API 29 よりも新しいとき)(IMEを指定する)
                        if (Build.VERSION.SDK_INT > 29) { // 26:OK 29:OK 33:OK 34:未試験
                            //println("\nLine:" + Throwable().stackTrace[0].lineNumber + " ★MainActivity★Build.VERSION.SDK_INT=${Build.VERSION.SDK_INT} Build.VERSION_CODES.O=${Build.VERSION_CODES.O} ★")
                            uYY.windowInsetsController?.show(WindowInsetsCompat.Type.ime())
                        }
                        val imm = activity?.getSystemService(INPUT_METHOD_SERVICE) as InputMethodManager
                        imm.showSoftInput(uYY, 0)
                    }
                }
                //Toast.makeText(this@MainActivity, uSS_text, Toast.LENGTH_SHORT).show()
            }

            //[元号]スピンナーが選択されなかったとき (削除してはいけない)
            override fun onNothingSelected(parent: AdapterView<*>?) {}
        }


        //[年](yy)のフォーカスが変化したとき ---------------------------------------------------- [年](yy)
        uYY.setOnFocusChangeListener { _, hasFocus -> //v, hasFocus

            //桁数が揃えられた日付を取得する
            val(yx, mx, dx) = setYmdLength(uix, uYY.text.toString(), uMM.text.toString(), uDD.text.toString())
            //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★setYmdLength 桁数-out★\n★uix=$uix\n★yx=$yx\n★mx=$mx\n★dx=$dx★")

            //フォーカスを持ったとき
            if ( hasFocus ){
                btnExec.isEnabled = false
                if( yx.isNotEmpty() ){ (uYY as EditText).setSelection(yx.length) }
            }

            //フォーカスが外れるとき
            else{
                //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★uYYhasFocus false★ yx=$yx mx=$mx cd=$dx ")

                //[年](yy)が入力されているとき
                if( yx.isNotEmpty() ) {
                    var rst = false

                    //西暦のとき
                    if ( uix == 1 ) {

                        //[年](yyyy)の妥当性を判定する
                        rst = du.checkYY(uix, minYear, "", yx)
                    }

                    //和暦のとき
                    else if ( uix >= 2 ){

                        //[年](yy)の妥当性を判定する
                        rst = du.checkYY(uix, minYear, yx, "")
                    }

                    //西暦年(yyyy)または和暦年(yy)が妥当なとき
                    if( rst ){
                        //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★FragmentEdtHowold year★ uix=$uix yx=$yx l=${yx.length} ★")

                        //[年](yy)(和暦では前ゼロなし)を再表示する
                        uYY.text = yx
                        uYY.setTextColor(Color.GRAY)

                        //月(mm)が入力されていないとき
                        if ( mx == "" ) {
                            //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★FragmentEdtHowold yx=$yx mx=$mx cd=$dx ")

                            //月(mm)に移動する
                            uMM.isEnabled = true
                            uMM.requestFocus()
                            //if (uMM.text.isNotEmpty()) { (uMM as EditText).setSelection(uMM.text.length) }
                        }

                        //月(mm)と日(dd)が入力されているとき
                        else if (mx.isNotEmpty() && dx.isNotEmpty()) {
                            //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★FragmentEdtHowold yx=$yx mx=$mx cd=$dx ")

                            //キーボードを閉じて[保存]ボタンを有効にする
                            closeKeyBord(view)
                            btnExec.isEnabled = true

                            //[保存]ボタンに移動する
                            btnExec.requestFocus()
                        }
                    }

                    //西暦年(yyyy)または和暦年(yy)が妥当でないとき
                    else{

                        //[年](yy)の文字色を赤にする
                        if( yx.isNotEmpty() ){

                            uYY.setTextColor(Color.RED)
                            (uYY as EditText).setSelection(uYY.text.length)
                            closeKeyBord(view) //文字入力を表示させない
                        }
                    }

                } //yx.isNotEmpty()
            } //focusout
        }

        //[年](yy)が入力されたとき
        uYY.addTextChangedListener(object : TextWatcher {
            override fun beforeTextChanged(s: CharSequence?, start: Int, count: Int, after: Int) {}
            override fun onTextChanged(s: CharSequence?, start: Int, before: Int, count: Int) {}
            override fun afterTextChanged(s: Editable?) {
                //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★FragmentEdtHowold TextWatcher★ uix=$uix s=${s?.length} ★")

                //西暦年(yyyy)または和暦年(yy)の桁数が満たされたとき
                if ((uix == 1 && s?.length!! == 4) || (uix >= 2 && s?.length!! == 2)) {
                    uMM.isEnabled = true
                    uMM.requestFocus()
                    if (uMM.text.isNotEmpty()) { (uMM as EditText).setSelection(uMM.text.length) }
                }
            } //afterTextChanged
        }) //[年](yy)が入力されたとき


        //[年](yy)からキーボードの完了(Done)ボタンのイベントを検知する (和暦1桁のときに必要となる)
        uYY.setOnEditorActionListener { _, actionId, _ -> //view, actionId, event

            //[年](yy)の入力が終了したとき
            if ( actionId == EditorInfo.IME_ACTION_DONE ) {
                //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★FragmentEdtHowold actionId★ uix=$uix uYY.text=${uYY.text} ★")

                if( uYY.text.toString().isNotEmpty() ) {
                    uMM.isEnabled = true
                    uMM.requestFocus()
                    if( uMM.text.isNotEmpty() ){ (uMM as EditText).setSelection(uMM.text.length) }
                }
            }
            return@setOnEditorActionListener true
        } //完了(Done)ボタンのとき


        //[月](mm)のフォーカスが変化したとき ---------------------------------------------------- [月](mm)
        uMM.setOnFocusChangeListener { _, hasFocus -> //v, hasFocus

            //入力された日付を取得する
            val(_, mx, dx) = setYmdLength(uix, uYY.text.toString(), uMM.text.toString(), uDD.text.toString())
            //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★setYmdLength 桁数★ uix=$uix yx=$yx mx=$mx dx=$dx ★")

            //フォーカスを持ったとき
            if ( hasFocus ){
                btnExec.isEnabled = false
                if( mx.isNotEmpty() ){ (uMM as EditText).setSelection(mx.length) }
            }
            //フォーカスが外れるとき
            else {
                //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★uMMhasFocus false★ uix=$uix yx=$yx mx=$mx dx=$dx ★")

                //[月](mm)が空でないとき
                if( mx.isNotEmpty() ){

                    //[月](mm)が妥当なとき
                    val mn = mx.toInt()
                    if ( mn in 1..12 ) { //mn>=1 && mn<=12

                        //[月](mm)を再表示する
                        uMM.text = mx
                        uMM.setTextColor(Color.GRAY)

                        //日(dd)に移動する
                        uDD.isEnabled = true
                        uDD.requestFocus()
                        if( dx.isNotEmpty() ){ (uDD as EditText).setSelection(dx.length) }
                    }
                    //[月](mm)が妥当でないとき
                    else {

                        //表示色を赤に変える
                        uMM.setTextColor(Color.RED)
                        (uMM as EditText).setSelection(mx.length)
                        closeKeyBord(view)
                    }
                } //mx.isNotEmpty()
            }
        }

        //[月](mm)が入力されたとき
        uMM.addTextChangedListener(object : TextWatcher {
            override fun beforeTextChanged(s: CharSequence?, start: Int, count: Int, after: Int) {}
            override fun onTextChanged(s: CharSequence?, start: Int, before: Int, count: Int) {}
            override fun afterTextChanged(s: Editable?) {

                //[月](mm)が2桁のとき
                if( s?.length== 2 ) {
                    //日(dd)に移動する
                    uDD.isEnabled = true
                    uDD.requestFocus()
                    if( uDD.text.isNotEmpty() ){ (uDD as EditText).setSelection(uDD.text.length) }
                }
            }
        }) //[月](mm)が入力されたとき


        //[月](mm)からキーボードの完了(Done)ボタンのイベントを検知する
        uMM.setOnEditorActionListener { _, actionId, _ -> //view, actionId, event

            //[月](mm)の入力が終了したとき
            if ( actionId == EditorInfo.IME_ACTION_DONE ) {
                //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★FragmentEdtHowold IME_ACTION_DONE★ uMM.text=${uMM.text} ★")

                //日(dd)に移動する
                if( uMM.text.toString().isNotEmpty() ) {
                    uDD.isEnabled = true
                    uDD.requestFocus()
                    if (uDD.text.isNotEmpty()) { (uDD as EditText).setSelection(uDD.text.length) }
                }
            } //[月](mm)の入力が(0桁または)1桁で終了したとき
            return@setOnEditorActionListener true
        } //完了(Done)ボタンのとき


        //[日](dd)のフォーカスが変化したとき ---------------------------------------------------- [日](dd)
        uDD.setOnFocusChangeListener { _, hasFocus -> //v, hasFocus

            //入力された日付を取得する
            val(yx, mx, dx) = setYmdLength(uix, uYY.text.toString(), uMM.text.toString(), uDD.text.toString())
            //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★setYmdLength 桁数★ uix=$uix yx=$yx mx=$mx dx=$dx ★")

            //フォーカスを持ったとき
            if ( hasFocus ){
                btnExec.isEnabled = false
                if( dx.isNotEmpty() ){ (uDD as EditText).setSelection(dx.length) }
            }
            //フォーカスが外れるとき
            else{
                //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★uDDhasFocus false★ uix=$uix yx=$yx mx=$mx dx=$dx ★")

                //[日](dd)が入力されたとき
                if( dx.isNotEmpty() ) {

                    //[日](dd)が妥当なとき
                    if( du.checkDD(mx, dx) ) {
                        //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★FragmentEdtHowold★ uix=$uix yx=$yx mx=$mx dx=$dx ★")

                        //[日](dd)を再表示する
                        uDD.text = dx
                        uDD.setTextColor(Color.GRAY)

                        //参照日付を生成する
                        val( refYMD, refJMD ) = setupYMD(uix, yx, mx, dx, iEra)

                        //干支を取得する
                        itmZodc = db.getZodiac(refYMD.replace("/",""))

                        //参照日付を表示する
                        dspRefRevs("up", uix, refYMD, refJMD, itmZodc)

                        //キーボードを閉じて[保存]ボタンを有効にする
                        closeKeyBord(view)
                        btnExec.isEnabled = true

                        //[終了日]スピンナーを有効にする
                        dSS.isEnabled = true
                        uTL.requestFocus() //dummy
                    }
                    //[日](dd)が妥当でないとき
                    else {
                        //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★FragmentEdtHowold★")
                        uDD.setTextColor(Color.RED)
                        (uDD as EditText).setSelection(dx.length)
                        closeKeyBord(view)
                    }
                }
            }
        }

        //[日](dd)が入力されたとき
        uDD.addTextChangedListener(object : TextWatcher {
            override fun beforeTextChanged(s: CharSequence?, start: Int, count: Int, after: Int) { }
            override fun onTextChanged(s: CharSequence?, start: Int, before: Int, count: Int) { }
            override fun afterTextChanged(s: Editable?) {

                //フォーカスを外す ([保存]または[終了日]の二択となる)
                if( s?.length==2) {
                    uTL.requestFocus() //dummy
                }
            }
        }) //[日](dd)が入力されたとき


        //[日](dd)からキーボードの完了(Done)ボタンのイベントを検知する
        uDD.setOnEditorActionListener { _, actionId, _ -> //view, actionId, event

            //[日](dd)の入力が終了したとき
            if ( actionId == EditorInfo.IME_ACTION_DONE ) {
                //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★FragmentEdtHowold IME_ACTION_DONE★ uDD.text=${uDD.text}")

                //[日](dd)が空でないとき
                if( uDD.text.toString().isNotEmpty() ) {

                    //フォーカスを外す ([保存]または[終了日]の二択となる)
                    uTL.requestFocus() //dummy
                }
            }
            return@setOnEditorActionListener true
        }

        //終了日:[元号]リスナーを実装する
        dSS.onItemSelectedListener = object : AdapterView.OnItemSelectedListener {

            //元号が選択されたとき
            @RequiresApi(Build.VERSION_CODES.R)
            override fun onItemSelected(parent: AdapterView<*>?, view: View?, position: Int, id: Long ) {
                //println("Line:"+ Throwable().stackTrace[0].lineNumber +" ★FragmentEdtHowold [終了日]リスナー \n position=$position \n itmRcid=$itmRcid \n itmName=$itmName \n itmUpnm=$itmUpnm \n itmDnnm=$itmDnnm \n itmBegn=$itmBegn \n itmTerm=$itmTerm")

                //[元号]スピンナーの選択値を読み取る
                dix = dSS.selectedItemPosition

                //元号が未選択または再選択されたとき(西暦･和暦または元号を変更したとき)
                if ( dix==0 || itmDnnm!=iEra[dix] ) { // 登録されている元号 != 開始日の元号(空白)
                    //println("Line:"+Throwable().stackTrace[0].lineNumber+" ★FragmentEdtHowold★ itmTerm=$itmTerm dix=$dix itmDnnm=$itmDnnm iEra[dix]=${iEra[dix]}")

                    //終了日の設定値をクリアして入力不可にする(再入力させる)
                  //btnExec.isEnabled = false
                    itmTerm = "8888/88/88"
                  //itmDnnm = ""
                    dYY.text = ""; dYY.isEnabled = false
                    dMM.text = ""; dMM.isEnabled = false
                    dDD.text = ""; dDD.isEnabled = false
                    dRR.text = "" //参照(反転)表示

                    //[終了日]が空白に変更されたとき
                    if ( dix==0 && itmDnnm!="" ) {
                        //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★FragmentEdtHowold★ itmTerm=$itmTerm dix=$dix itmDnnm=$itmDnnm iEra[dix]=${iEra[dix]}")

                        //[保存]ボタンを有効にする (終了日を削除するため)
                        btnExec.isEnabled = true
                    }
                }

                //有効な値が選択されたとき
                if ( dix!=0 ) {

                    itmDnnm = iEra[dix] // "  ","西暦","明治","大正","昭和","平成","令和"
                    //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★FragmentEdtHowold itmDnnm=$itmDnnm")

                    //年(yy)が空のとき
                    if ( dYY.text.toString() == "" ) {
                        //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★FragmentEdtHowold dYY.text=${dYY.text}")

                        //[年]の入力を有効にする
                        dYY.isEnabled = true
                        dYY.requestFocus()

                        //キーボードを開く (Android 10.0 : Quince API 29 よりも新しいとき)(IMEを指定する)
                        if (Build.VERSION.SDK_INT > 29) { // 26:OK 29:OK 33:OK 34:未試験
                            //println("\nLine:" + Throwable().stackTrace[0].lineNumber + " ★MainActivity★Build.VERSION.SDK_INT = ${Build.VERSION_CODES.O} ★")
                            dYY.windowInsetsController?.show(WindowInsetsCompat.Type.ime())
                        }
                        val imm = activity?.getSystemService(INPUT_METHOD_SERVICE) as InputMethodManager
                        imm.showSoftInput(dYY, 0)
                    }
                    //Toast.makeText(this@MainActivity, dGG_text, Toast.LENGTH_SHORT).show()
                }
            }

            //[元号]スピンナーが選択されなかったとき (削除してはいけない)
            override fun onNothingSelected(parent: AdapterView<*>?) {}
        }

        //[年](yy)のフォーカスが変化したとき ---------------------------------------------------- [年](yy)
        dYY.setOnFocusChangeListener { _, hasFocus -> //v, hasFocus

            //入力された日付を取得する
            val(yx, mx, dx) = setYmdLength(dix, dYY.text.toString(), dMM.text.toString(), dDD.text.toString())
            //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★setYmdLength 桁数-out★\n★uix=$uix\n★yx=$yx\n★mx=$mx\n★dx=$dx★")

            //フォーカスを持ったとき
            if ( hasFocus ){
                btnExec.isEnabled = false
                if( yx.isNotEmpty() ){ (dYY as EditText).setSelection(yx.length) }
            }
            //フォーカスが外れるとき
            else{
                //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★FragmentEdtHowold★ yx=$yx mx=$mx dx=$dx ★")

                //[年](yy)が入力されているとき
                if( yx.isNotEmpty() ) {
                    var rst = false

                    //西暦のとき
                    if ( dix == 1 ) {

                        //[年](yyyy)の妥当性を判定する
                        rst = du.checkYY(dix, minYear, "", yx)
                    }

                    //和暦のとき
                    else if ( dix >= 2 ){

                        //[年](yy)の妥当性を判定する
                        rst = du.checkYY(dix, minYear, yx, "")
                    }

                    //西暦年(yyyy)または和暦年(yy)が妥当なとき
                    if( rst ){
                        //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★FragmentEdtHowold year★ dix=$dix yx=$yx l=${yx.length} ★")

                        //[年](yy)(和暦では前ゼロなし)を再表示する
                        dYY.text = yx
                        dYY.setTextColor(Color.GRAY)

                        //月(mm)が入力されていないとき
                        if ( mx == "" ) {
                            //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★FragmentEdtHowold yx=$yx mx=$mx cd=$dx ")

                            //月(mm)に移動する
                            dMM.isEnabled = true
                            dMM.requestFocus()
                            //if (dMM.text.isNotEmpty()) { (dMM as EditText).setSelection(dMM.text.length) }
                        }

                        //月(mm)と日(dd)が入力されているとき
                        else if (mx.isNotEmpty() && dx.isNotEmpty()) {
                            //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★FragmentEdtHowold yx=$yx mx=$mx cd=$dx ")

                            //キーボードを閉じて[保存]ボタンを有効にする
                            closeKeyBord(view)
                            btnExec.isEnabled = true

                            //[保存]ボタンに移動する
                            btnExec.requestFocus()
                        }
                    }

                    //西暦年(yyyy)または和暦年(yy)が妥当でないとき
                    else{

                        //[年](yy)の文字色を赤にする
                        if( yx.isNotEmpty() ){

                            dYY.setTextColor(Color.RED)
                            (dYY as EditText).setSelection(dYY.text.length)
                            closeKeyBord(view) //文字入力を表示させない
                        }
                    }
                } //cy.isNotEmpty()
            }
        }

        //[年](yy)が入力されたとき
        dYY.addTextChangedListener(object : TextWatcher {
            override fun beforeTextChanged(s: CharSequence?, start: Int, count: Int, after: Int) {}
            override fun onTextChanged(s: CharSequence?, start: Int, before: Int, count: Int) {}
            override fun afterTextChanged(s: Editable?) {
                //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★FragmentEdtHowold TextWatcher★ dix=$dix s=${s?.length} ★")

                //西暦年(yyyy)または和暦年(yy)の桁数が満たされたとき
                if ( (dix == 1 && s?.length == 4) || (dix >= 2 && s?.length == 2) ) {
                    dMM.isEnabled = true
                    dMM.requestFocus()
                    if (dMM.text.isNotEmpty()) { (dMM as EditText).setSelection(dMM.text.length) }
                }
            } //afterTextChanged
        }) //[年](yy)が入力されたとき


        //[年](yy)からキーボードの完了(Done)ボタンのイベントを検知する (和暦のときに必要となる)
        dYY.setOnEditorActionListener { _, actionId, _ -> //view, actionId, event

            //[年](yy)の入力が終了したとき
            if ( actionId == EditorInfo.IME_ACTION_DONE ) {
                //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★FragmentEdtHowold actionId★ dix=$dix dMM.text=${dMM.text} ★")

                if( dYY.text.toString().isNotEmpty() ) {
                    dMM.isEnabled = true
                    dMM.requestFocus()
                    if( dMM.text.isNotEmpty() ){ (dMM as EditText).setSelection(dMM.text.length) }
                }
            }
            return@setOnEditorActionListener true
        } //完了(Done)ボタンのとき


        //[月](mm)のフォーカスが変化したとき ---------------------------------------------------- [月](mm)
        dMM.setOnFocusChangeListener { _, hasFocus -> //v, hasFocus

            //入力された日付を取得する
            val(_, mx, dx) = setYmdLength(dix, dYY.text.toString(), dMM.text.toString(), dDD.text.toString())
            //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★setYmdLength 桁数★ dix=$dix yx=$yx mx=$mx dx=$dx ★")

            //フォーカスを持ったとき
            if ( hasFocus ){
                btnExec.isEnabled = false
                if( mx.isNotEmpty() ){ (dMM as EditText).setSelection(mx.length) }
            }
            //フォーカスが外れるとき
            else {
                //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★フォーカスが外れるとき★ dix=$dix yx=$yx mx=$mx dx=$dx ★")

                //月(mm)が空でないとき
                if( mx.isNotEmpty() ) {

                    //[月](mm)が妥当なとき
                    val mn = mx.toInt()
                    if ( mn in 1..12 ) { //mn>=1 && mn<=12

                        //[月](mm)を再表示する
                        dMM.text = mx
                        dMM.setTextColor(Color.GRAY)

                        //日(dd)に移動する
                        dDD.isEnabled = true
                        dDD.requestFocus()
                        if( dx.isNotEmpty() ){ (dDD as EditText).setSelection(dx.length) }
                    }
                    //[月](mm)が妥当でないとき
                    else {

                        //表示色を赤に変える
                        dMM.setTextColor(Color.RED)
                        (dMM as EditText).setSelection(mx.length)
                        closeKeyBord(view)
                    }
                } //mx.isNotEmpty()
            }
        }

        //[月](mm)が入力されたとき
        dMM.addTextChangedListener(object : TextWatcher {
            override fun beforeTextChanged(s: CharSequence?, start: Int, count: Int, after: Int) {}
            override fun onTextChanged(s: CharSequence?, start: Int, before: Int, count: Int) {}
            override fun afterTextChanged(s: Editable?) {

                //[月](mm)が2桁のとき
                if( s?.length== 2 ) {
                    //日(dd)に移動する
                    dDD.isEnabled = true
                    dDD.requestFocus()
                    if( dDD.text.isNotEmpty() ){ (dDD as EditText).setSelection(dDD.text.length) }
                }
            }
        }) //[月](mm)が入力されたとき


        //[月](mm)からキーボードの完了(Done)ボタンのイベントを検知する
        dMM.setOnEditorActionListener { _, actionId, _ -> //view, actionId, event

            //[月](mm)の入力が終了したとき
            if ( actionId == EditorInfo.IME_ACTION_DONE ) {
                //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★FragmentEdtHowold IME_ACTION_DONE★ dMM.text=${dMM.text} ★")

                //日(dd)に移動する
                if( dMM.text.toString().isNotEmpty() ) {
                    dDD.isEnabled = true
                    dDD.requestFocus()
                    if (dDD.text.isNotEmpty()) { (dDD as EditText).setSelection(dDD.text.length) }
                }
            } //[月](mm)の入力が(0桁または)1桁で終了したとき
            return@setOnEditorActionListener true
        } //完了(Done)ボタンのとき


        //[日](dd)のフォーカスが変化したとき ---------------------------------------------------- [日](dd)
        dDD.setOnFocusChangeListener { _, hasFocus -> //v, hasFocus

            //入力された日付を取得する
            val(yx, mx, dx) = setYmdLength(dix, dYY.text.toString(), dMM.text.toString(), dDD.text.toString())
            //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★setYmdLength 桁数★ dix=$dix yx=$yx mx=$mx dx=$dx ★")

            //フォーカスを持ったとき
            if ( hasFocus ){
                btnExec.isEnabled = false
                if( dx.isNotEmpty() ){ (dDD as EditText).setSelection(dx.length) }
            }
            //フォーカスが外れるとき
            else{
                //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★uDDhasFocus false★ dix=$dix yx=$yx mx=$mx dx=$dx ★")

                //[日](dd)が入力されたとき
                if( dx.isNotEmpty() ) {

                    //[日](dd)が妥当なとき
                    if( du.checkDD(mx, dx) ) {
                        //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★FragmentEdtHowold★ dix=$dix yx=$yx mx=$mx dx=$dx ★")

                        //[日](dd)を再表示する
                        dDD.text = dx
                        dDD.setTextColor(Color.GRAY)

                        //参照日付を生成する
                        val(refYMD, refJMD) = setupYMD(dix, yx, mx, dx, iEra)
                        //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★FragmentEdtHowold★ dix=$dix refYMD=$refYMD refJMD=$refJMD ★")

                        //参照日付を表示する
                        dspRefRevs("dn", dix, refYMD, refJMD, itmZodc)

                        //キーボードを閉じて[保存]ボタンを有効にする
                        closeKeyBord(view)
                        btnExec.isEnabled = true

                        //フォーカスを外す
                        uTL.requestFocus() //dummy
                    }
                    //[日](dd)が妥当でないとき
                    else {
                        //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★FragmentEdtHowold★")
                        dDD.setTextColor(Color.RED)
                        (dDD as EditText).setSelection(dx.length)
                        closeKeyBord(view)
                    }

                    //開始日(ymd)と終了日(ymd)の前後関係をチェックする
                    var uYMD = 0
                    var dYMD = 0
                    if( uix==1 ) {
                        val(uy, um, ud) = setYmdLength(uix, uYY.text.toString(), uMM.text.toString(), uDD.text.toString())
                        uYMD = "$uy$um$ud".toInt()
                    }else if( uix>=2 ){
                        uYMD = view.findViewById<TextView>(R.id.refRevs_up)?.text.toString().replace("/","").toInt()
                    }
                    if( dix==1 ) {
                        val(dy, dm, dd) = setYmdLength(dix, dYY.text.toString(), dMM.text.toString(), dDD.text.toString())
                        dYMD = "$dy$dm$dd".toInt()
                    }else if( dix>=2 ){
                        dYMD = view.findViewById<TextView>(R.id.refRevs_dn)?.text.toString().replace("/","").toInt()
                    }
                    //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★FragmentEdtHowold★ uYMD=$uYMD dYMD=$dYMD")

                    //開始日が終了日よりも後のとき
                    if( uYMD > dYMD ){
                        btnExec.isEnabled = false
                        view.findViewById<TextView>(R.id.lblBegn).setTextColor(Color.RED)
                        view.findViewById<TextView>(R.id.lblTerm).setTextColor(Color.RED)
                        Toast.makeText(activity, "開始日と終了日との前後関係が間違ってます。", Toast.LENGTH_LONG).show()

                        //終了日の設定値をクリアして再入力させる
                        dSS.setSelection(0)

                    }else{
                        btnExec.isEnabled = true
                        view.findViewById<TextView>(R.id.lblBegn).setTextColor(Color.GRAY)
                        view.findViewById<TextView>(R.id.lblTerm).setTextColor(Color.GRAY)
                    }
                }
            }
        }

        //[日](dd)が入力されたとき
        dDD.addTextChangedListener(object : TextWatcher {
            override fun beforeTextChanged(s: CharSequence?, start: Int, count: Int, after: Int) { }
            override fun onTextChanged(s: CharSequence?, start: Int, before: Int, count: Int) { }
            override fun afterTextChanged(s: Editable?) {

                //フォーカスを外す ([保存]または[取消]の二択となる)
                if( s?.length==2) {
                    uTL.requestFocus() //dummy
                }
            }
        }) //[日](dd)が入力されたとき


        //[日](dd)からキーボードの完了(Done)ボタンのイベントを検知する
        dDD.setOnEditorActionListener { _, actionId, _ -> //view, actionId, event

            //[日](dd)の入力が終了したとき
            if ( actionId == EditorInfo.IME_ACTION_DONE ) {
                //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★FragmentEdtHowold IME_ACTION_DONE★ dDD.text=${dDD.text}")

                //[日](dd)が空でないとき
                if( dDD.text.toString().isNotEmpty() ) {

                    //フォーカスを外す ([保存]または[取消]の二択となる)
                    uTL.requestFocus() //dummy
                }
            }
            return@setOnEditorActionListener true
        }

        // ↑↑↑↑↑ ここまで日付ビュー ↑↑↑↑↑ ===== ↓↓↓↓↓ ここから機能ボタン ↓↓↓↓↓ ================================

        //[取消]ボタンがクリックされたとき
        view.findViewById<Button>(R.id.btnCanc).setOnClickListener {
            //println("Line:"+ Throwable().stackTrace[0].lineNumber +" Button: [取消]")

            //プログレスバーを表示する
            view.findViewById<View>(R.id.howoldEdtProgressBar).visibility = View.VISIBLE

            //すぐに年齢リストに切り替える
            restartHowold("btnHowoldCanc")
        }

        //[保存]ボタンがクリックされたとき
        btnExec.setOnClickListener {
            //println("Line:"+ Throwable().stackTrace[0].lineNumber +" ★FragmentEdtHowold Button:[保存] itmBegn=$itmBegn itmTerm=$itmTerm")

            //プログレスバーを表示する
            view.findViewById<View>(R.id.howoldEdtProgressBar).visibility = View.VISIBLE

            //編集されたデータを読み取る
            val txtName = uNM.text.toString()
            val txtKana = uKN.text.toString()

            //[元号]スピンナーの選択値を読み取る
            val upnm = iEra[uix]
            var dnnm = iEra[dix]; if (dix==0) { dnnm="" }

            //開始日を読み取る
            var txtBegn = ""
            if( uix==1 ){ //西暦
                txtBegn = "${uYY.text}/${uMM.text}/${uDD.text}"
            }
            else if( uix>=2 ){ //和暦
                val(yn, _, mn, dn) = du.changeJpnToAnno(upnm, "${uYY.text}","${uMM.text}","${uDD.text}", iEra) //yn, abbr
                txtBegn = "$yn/$mn/$dn"
            }
            //println("Line:"+ Throwable().stackTrace[0].lineNumber +" ★FragmentEdtHowold txtBegn=$txtBegn ★")

            //終了日を読み取る
            var txtTerm = ""
            if( dix==0 ){ //空
                txtTerm = ""
            }
            else if( dix==1 ){ //西暦
                txtTerm = "${dYY.text}/${dMM.text}/${dDD.text}"
            }
            else if( dix>=2 ){ //和暦
                val(yn, _, mn, dn) = du.changeJpnToAnno(dnnm, "${dYY.text}","${dMM.text}","${dDD.text}", iEra) //yn, abbr
                txtTerm = "$yn/$mn/$dn"
            }
            //println("Line:"+ Throwable().stackTrace[0].lineNumber +" ★FragmentEdtHowold txtTerm=$txtTerm ★")

            //追加: Rcidが空のとき =============
            if (itmRcid == "") {
                //println("Line:"+ Throwable().stackTrace[0].lineNumber +" ★FragmentEdtHowold Button: [追加] \n txtName=$txtName \n txtKana=$txtKana \n upnm=$upnm \n dnnm=$dnnm \n txtBegn=$txtBegn \n txtTerm=$txtTerm")

                //レコードを追加する
                db.insertHowoldData(txtName, txtKana, upnm, dnnm, txtBegn, txtTerm)
            }

            //更新: rcidが空でないとき ==========
            else {

                //保持したrcidを読み取る
                val rcid = itmRcid.toInt()
                //println("Line:"+ Throwable().stackTrace[0].lineNumber +" ★FragmentEdtHowold Button: [更新] \n rcid=$rcid \n txtName=$txtName \n txtKana=$txtKana \n upnm=$upnm \n dnnm=$dnnm \n txtBegn=$txtBegn \n txtTerm=$txtTerm")

                //レコードを更新する
                db.updateHowoldData(rcid, txtName, txtKana, upnm, dnnm, txtBegn, txtTerm)
            }

            //年齢データを追加または更新してから年齢フラグメントに切り替える
            restartHowold("btnHowoldExec")
        }

        //[削除]ボタンがクリックされたとき
        view.findViewById<Button>(R.id.btnDele).setOnClickListener {
            //println("Line:"+ Throwable().stackTrace[0].lineNumber +" ★FragmentEdtHowold Button: [削除] itmRcid=$itmRcid ")

            //プログレスバーを表示する
            view.findViewById<View>(R.id.howoldEdtProgressBar).visibility = View.VISIBLE

            //保持したrcidを読み取る
            val rcid = itmRcid.toInt()
            //println("Line:"+ Throwable().stackTrace[0].lineNumber +" ★FragmentEdtHowold Button: [削除] itmRcid=$itmRcid rcid=$rcid")

            //レコードを削除する
            db.deleteHowoldData(rcid)

            //年齢データを削除してから年齢フラグメントに切り替える
            restartHowold("btnHowoldDele")
        }

        //println("Line:"+ Throwable().stackTrace[0].lineNumber +" ★FragmentEdtHowold onViewCreated 末尾")
    } //onViewCreated (setupData)
/*
    //オブジェクトを破棄する
    @RequiresApi(Build.VERSION_CODES.O)
    override fun onDestroy() {
        super.onDestroy()
        println("Line:"+ Throwable().stackTrace[0].lineNumber +" ★FragmentEdtHowold ★★★ onDestroy ★★★" )
    }
*/
    //==============================================================================================

    //==============================================================================================

    //【関数】日付を表示する (参照日付は西暦と和暦とを逆に表示する)
    @SuppressLint("SetTextI18n")
    fun dspDateValue(ud:String) { //ud upside/downside 開始/終了
        //println("Line:" + Throwable().stackTrace[0].lineNumber +" ★dspDateValue ud=$ud★")

        //参照日付のために日付の単位を取得する(年月日)
        val lY = resources.getString(R.string.lblYear) //年
        val lM = resources.getString(R.string.lblMoth) //月
        val lD = resources.getString(R.string.lblXday) //日

        //日付のビューを取得する
        val uSS = view?.findViewById<Spinner>(R.id.spinner_up)
      //val dSS = view?.findViewById<Spinner>(R.id.spinner_dn) 開始日の入力後に有効となる
        val upY = view?.findViewById<TextView>(R.id.inpYY_up)
        val upM = view?.findViewById<TextView>(R.id.inpMM_up)
        val upD = view?.findViewById<TextView>(R.id.inpDD_up)
        val dnY = view?.findViewById<TextView>(R.id.inpYY_dn)
        val dnM = view?.findViewById<TextView>(R.id.inpMM_dn)
        val dnD = view?.findViewById<TextView>(R.id.inpDD_dn)
        //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★dspDateValue★\n★dnY=$dnY\n★dnM=$dnM\n★dnD=$dnD" )

        //日付を取得する
        val gix:Int
        val yy:String
        val mm:String
        val dd:String
        if( ud=="up" ) {
          gix = uix
          yy = itmBegn.substring(0, 4) //yyyy
          mm = itmBegn.substring(5, 7) //mm
          dd = itmBegn.substring(8, 10) //dd

        }else{
          gix = dix
          yy = itmTerm.substring(0, 4) //yyyy
          mm = itmTerm.substring(5, 7) //mm
          dd = itmTerm.substring(8, 10) //dd
        }

        //西暦年を和暦年に変換する
        val (gg, jy, _) = du.changeAnnoToJpn(yy, mm, dd, iEra) //元号,和暦年,略号
        val jm = mm.toInt().toString()
        val jd = dd.toInt().toString()

        //開始日のとき
        if ( ud == "up" ) {

            //氏名(氏名(名称)に空白を含んでいる)とき
            val chkName = itmName.replace("\\s".toRegex(), "")
            if ( itmName.length != chkName.length ) {
                //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★dspDateValue★ itmBegn=$itmBegn itmTerm=$itmTerm itmName=$itmName chkName=$chkName itmZodc=$itmZodc")

                //【関数】生まれ年から干支を取得する
                itmZodc = db.getZodiac("$yy/$mm/$dd") //"$stem($ruby)/$zodc" 十干(よみ)/十二支
            }
            //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★dspDateValue★ itmBegn=$itmBegn itmTerm=$itmTerm itmName=$itmName chkName=$chkName itmZodc=$itmZodc")

            //新規登録のとき
            if( itmBegn == "8888/88/88" ) {

                //元号スピンナーを有効にする(新規登録の直後に編集に入ると有効になっていない)
                uSS?.isEnabled = false
                uSS?.setSelection(gix)
            }

            //西暦のとき
            if ( gix == 1 ) {
               //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★dspDateValue★ gix=$gix yy=$yy mm=$mm dd=$dd")

                //西暦の年月日を表示する
                upY?.text = yy
                upM?.text = mm //前ゼロあり
                upD?.text = dd
            }
            //和暦のとき
            else {
                //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★dspDateValue★ gix=$gix jy=$jy jm=$jm jd=$jd itmZodc=$itmZodc")

                //和暦の年月日を表示する
                upY?.text = jy
                upM?.text = jm //前ゼロなし
                upD?.text = jd
            }

            //参照日付を表示する
            dspRefRevs(ud, gix, "$yy/$mm/$dd", "$gg$jy$lY$jm$lM$jd$lD", itmZodc)

        } //ex == "up"

        //終了日のとき
        else{ //ud == "dn"

            //西暦のとき
            if ( gix == 1 ) {
                //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★dspDateValue★ gix=$gix yy=$yy mm=$mm dd=$dd")

                //西暦の年月日を表示する
                dnY?.text = yy
                dnM?.text = mm //前ゼロあり
                dnD?.text = dd
            }
            //和暦のとき
            else {
               //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★dspDateValue★ gix=$gix jy=$jy jm=$jm jd=$jd")

                //和暦の年月日を表示する
                dnY?.text = jy
                dnM?.text = jm //前ゼロなし
                dnD?.text = jd
            }

            //参照日付を表示する
            dspRefRevs(ud, gix,"$yy/$mm/$dd", "$gg$jy$lY$jm$lM$jd$lD", "")

        } //ex == "dn"
    } //dspDateValue

    //参照日付を表示する
    private fun dspRefRevs(ud:String, gix:Int, ymd:String, jmd:String, zc:String) { //ud upside開始/downside終了
        //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★dspDateValue★ ud=$ud gix=$gix jmd=$ymd jmd=$jmd zc=$zc")

        //開始日のとき
        if( ud == "up" ) {

            //西暦のとき
            if( gix == 1 ){

                //参照日付に前ゼロなしの和暦を表示する
                view?.findViewById<TextView>(R.id.refRevs_up)?.text = jmd
            }
            //和暦のとき
            else{

                //参照日付に前ゼロありの西暦を表示する
                view?.findViewById<TextView>(R.id.refRevs_up)?.text = ymd

                //干支を付加する
                val zoc = zc.split("/")
                view?.findViewById<TextView>(R.id.refStem_up)?.text = zoc[0] //十干よみ
                view?.findViewById<TextView>(R.id.refZodc_up)?.text = zoc[1] //干支
            }
        }
        //終了日のとき
        else {

            //西暦のとき
            if( gix == 1 ){

                //参照日付に前ゼロなしの和暦を表示する
                view?.findViewById<TextView>(R.id.refRevs_dn)?.text = jmd
            }
            //和暦のとき
            else{

                //参照日付に前ゼロありの西暦を表示する
                view?.findViewById<TextView>(R.id.refRevs_dn)?.text = ymd
            }
        }
    }

    //入力された日付(西暦･和暦)を補完して参照日付を生成する
    private fun setupYMD(gix:Int, yx:String, mx:String, dx:String, iEra:Array<String>):Pair<String, String>{
        //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★dspDateValue★ gix=$gix yx=$yx mx=$mx dx=$dx")

        //参照日付の変数を準備する
        val refYMD:String
        val refJMD:String

        //参照日付のために日付の単位を取得する(年月日)
        val lY = resources.getString(R.string.lblYear) //年
        val lM = resources.getString(R.string.lblMoth) //月
        val lD = resources.getString(R.string.lblXday) //日

        //西暦のとき
        if( gix==1 ){

            //入力された日付(西暦)を和暦に変換する
            val(gx, jy, _) = du.changeAnnoToJpn(yx, mx, dx, iEra) //元号,和暦年,略号

            //参照日付を生成する
            refYMD = "$yx/$mx/$dx"
            refJMD = "$gx$jy$lY${mx.toInt()}$lM${dx.toInt()}$lD"
        }
        //和暦のとき
        else{
            //入力された日付(和暦)を西暦に変換する
            val (yy, _, mm, dd) = du.changeJpnToAnno(iEra[gix], yx, mx, dx, iEra) //西暦年,和暦略号,月,日

            //参照日付を生成する
            refYMD = "$yy/$mm/$dd"
            refJMD = "${iEra[gix]}$yx$lY$mx$lM$dx$lD"
        }
        return Pair(refYMD, refJMD)
    }

    //西暦または和暦の日付を読み取って日付の桁数をそろえる (空白はそのまま返す)
    private fun setYmdLength(sx:Int, yy:String, mm:String, dd:String): Array<String>{
        //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★setYmdLength 桁数-in ★ sx=$sx yy=$yy mm=$mm dd=$dd ★")

        //sx: select index 0,1,2...
        var yx = ""
        var mx = ""
        var dx = ""

        //西暦のとき
        if( sx == 1 ){

            //月(mm)と日(dd)に前ゼロを付けて2桁にする
            yx = yy
            mx = mm; if( mm.length == 1 ){ mx = "0$mm" }
            dx = dd; if( dd.length == 1 ){ dx = "0$dd" }
        }
        //和暦のとき
        if( sx >= 2 ){

            //年月日の前ゼロを除去する
            if( yy != "" ){ yx = yy.toInt().toString() }
            if( mm != "" ){ mx = mm.toInt().toString() }
            if( dd != "" ){ dx = dd.toInt().toString() }
        }
        //println("Line:" + Throwable().stackTrace[0].lineNumber + " ★setYmdLength 桁数-out★ sx=$sx yx=$yx mx=$mx dx=$dx ★")
        return arrayOf(yx, mx, dx)
    }

    //==============================================================================================

    //==============================================================================================
    //【関数】キーボードを閉じる
    private fun closeKeyBord(view: View?): Boolean {
        //println("Line:"+Throwable().stackTrace[0].lineNumber+" ★FragmentEdtHowold★closeKeyBord★")

        val imm = context?.getSystemService(INPUT_METHOD_SERVICE) as InputMethodManager
        if (view != null) {
            imm.hideSoftInputFromWindow(view.windowToken, 0)
        }
        return true
    } //fun closeKeyBord
/*
    //キーボードを開く
    private fun <windowInsetsController> openKeyBord(view: View?, field:Any): Boolean {
        //println("Line:"+Throwable().stackTrace[0].lineNumber+" ★FragmentEdtHowold★openKeyBord★")

        //Android 8.0 : Oreo 26 でないとき
        if (Build.VERSION.SDK_INT != Build.VERSION_CODES.O) {
            //println("\nLine:" + Throwable().stackTrace[0].lineNumber + " ★MainActivity★Build.VERSION.SDK_INT = ${Build.VERSION_CODES.O} ★")

            field.show(WindowInsetsCompat.Type.ime())
        }
        val imm = activity?.getSystemService(INPUT_METHOD_SERVICE) as InputMethodManager
        imm.showSoftInput(field, 0)

        return true
    }
*/
    //==============================================================================================
    //【関数】Fragmentを再スタートする
    private fun restartHowold(btn:String){
        //println("Line:"+ Throwable().stackTrace[0].lineNumber +" ★FragmentEdtHowold ★★★ restart ★★★ btn=$btn" )

        //引数をセットする
        val bundle = Bundle()
        bundle.putString("trns", btn)
        //println("Line:"+ Throwable().stackTrace[0].lineNumber +" ★FragmentDspHowold bundle=$bundle★" )

        //[年齢リスト]に戻る
        val dspHowoldFragment = FragmentDspHowold()
        dspHowoldFragment.arguments = bundle

        parentFragmentManager.beginTransaction()
          //.add(R.id.container, FragmentDspHowold())
            .replace(R.id.container, dspHowoldFragment)
            .commit()
    } //fun restartHowold

} //Fragment_Edt_Howold
